@@ -7,17 +7,19 @@ import os
 import sys
 import traceback
 import warnings
-import http.cookies
-import urllib.parse
+
+from multidict import CIMultiDict, MultiDict, MultiDictProxy, istr
+from yarl import URL
 
 import aiohttp
-from .client_reqrep import ClientRequest, ClientResponse
-from .errors import WSServerHandshakeError
-from .multidict import MultiDictProxy, MultiDict, CIMultiDict, upstr
-from .websocket import WS_KEY, WebSocketParser, WebSocketWriter
-from .websocket_client import ClientWebSocketResponse
-from . import hdrs
 
+from . import hdrs, helpers
+from ._ws_impl import WS_KEY, WebSocketParser, WebSocketWriter
+from .client_reqrep import ClientRequest, ClientResponse
+from .client_ws import ClientWebSocketResponse
+from .cookiejar import CookieJar
+from .errors import WSServerHandshakeError
+from .helpers import Timeout
 
 __all__ = ('ClientSession', 'request', 'get', 'options', 'head',
            'delete', 'post', 'put', 'patch', 'ws_connect')
@@ -35,7 +37,9 @@ class ClientSession:
                  headers=None, skip_auto_headers=None,
                  auth=None, request_class=ClientRequest,
                  response_class=ClientResponse,
-                 ws_response_class=ClientWebSocketResponse):
+                 ws_response_class=ClientWebSocketResponse,
+                 version=aiohttp.HttpVersion11,
+                 cookie_jar=None):
 
         if connector is None:
             connector = aiohttp.TCPConnector(loop=loop)
@@ -50,15 +54,26 @@ class ClientSession:
         if loop.get_debug():
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
-        self._cookies = http.cookies.SimpleCookie()
+        if not loop.is_running():
+            warnings.warn("Creating a client session outside of coroutine is "
+                          "a very dangerous idea", ResourceWarning,
+                          stacklevel=2)
+            context = {'client_session': self,
+                       'message': 'Creating a client session outside '
+                       'of coroutine'}
+            if self._source_traceback is not None:
+                context['source_traceback'] = self._source_traceback
+            loop.call_exception_handler(context)
+
+        if cookie_jar is None:
+            cookie_jar = CookieJar(loop=loop)
+        self._cookie_jar = cookie_jar
 
-        # For Backward compatability with `share_cookies` connectors
-        if connector._share_cookies:
-            self._update_cookies(connector.cookies)
         if cookies is not None:
-            self._update_cookies(cookies)
+            self._cookie_jar.update_cookies(cookies)
         self._connector = connector
         self._default_auth = auth
+        self._version = version
 
         # Convert to list of tuples
         if headers:
@@ -67,7 +82,7 @@ class ClientSession:
             headers = CIMultiDict()
         self._default_headers = headers
         if skip_auto_headers is not None:
-            self._skip_auto_headers = frozenset([upstr(i)
+            self._skip_auto_headers = frozenset([istr(i)
                                                  for i in skip_auto_headers])
         else:
             self._skip_auto_headers = frozenset()
@@ -88,38 +103,9 @@ class ClientSession:
                 context['source_traceback'] = self._source_traceback
             self._loop.call_exception_handler(context)
 
-    def request(self, method, url, *,
-                params=None,
-                data=None,
-                headers=None,
-                skip_auto_headers=None,
-                auth=None,
-                allow_redirects=True,
-                max_redirects=10,
-                encoding='utf-8',
-                version=aiohttp.HttpVersion11,
-                compress=None,
-                chunked=None,
-                expect100=False,
-                read_until_eof=True):
+    def request(self, method, url, **kwargs):
         """Perform HTTP request."""
-        return _RequestContextManager(
-            self._request(
-                method,
-                url,
-                params=params,
-                data=data,
-                headers=headers,
-                skip_auto_headers=skip_auto_headers,
-                auth=auth,
-                allow_redirects=allow_redirects,
-                max_redirects=max_redirects,
-                encoding=encoding,
-                version=version,
-                compress=compress,
-                chunked=chunked,
-                expect100=expect100,
-                read_until_eof=read_until_eof))
+        return _RequestContextManager(self._request(method, url, **kwargs))
 
     @asyncio.coroutine
     def _request(self, method, url, *,
@@ -131,19 +117,26 @@ class ClientSession:
                  allow_redirects=True,
                  max_redirects=10,
                  encoding='utf-8',
-                 version=aiohttp.HttpVersion11,
+                 version=None,
                  compress=None,
                  chunked=None,
                  expect100=False,
-                 read_until_eof=True):
+                 read_until_eof=True,
+                 proxy=None,
+                 proxy_auth=None,
+                 timeout=5*60):
+
+        if version is not None:
+            warnings.warn("HTTP version should be specified "
+                          "by ClientSession constructor", DeprecationWarning)
+        else:
+            version = self._version
 
         if self.closed:
             raise RuntimeError('Session is closed')
 
         redirects = 0
         history = []
-        if not isinstance(method, upstr):
-            method = upstr(method)
 
         # Merge with default headers and transform to CIMultiDict
         headers = self._prepare_headers(headers)
@@ -160,18 +153,28 @@ class ClientSession:
         skip_headers = set(self._skip_auto_headers)
         if skip_auto_headers is not None:
             for i in skip_auto_headers:
-                skip_headers.add(upstr(i))
+                skip_headers.add(istr(i))
+
+        if proxy is not None:
+            proxy = URL(proxy)
 
         while True:
+            url = URL(url).with_fragment(None)
+
+            cookies = self._cookie_jar.filter_cookies(url)
+
             req = self._request_class(
                 method, url, params=params, headers=headers,
                 skip_auto_headers=skip_headers, data=data,
-                cookies=self.cookies, encoding=encoding,
+                cookies=cookies, encoding=encoding,
                 auth=auth, version=version, compress=compress, chunked=chunked,
                 expect100=expect100,
-                loop=self._loop, response_class=self._response_class)
+                loop=self._loop, response_class=self._response_class,
+                proxy=proxy, proxy_auth=proxy_auth, timeout=timeout)
 
-            conn = yield from self._connector.connect(req)
+            with Timeout(timeout, loop=self._loop):
+                conn = yield from self._connector.connect(req)
+            conn.writer.set_tcp_nodelay(True)
             try:
                 resp = req.send(conn.writer, conn.reader)
                 try:
@@ -186,10 +189,7 @@ class ClientSession:
             except OSError as exc:
                 raise aiohttp.ClientOSError(*exc.args) from exc
 
-            self._update_cookies(resp.cookies)
-            # For Backward compatability with `share_cookie` connectors
-            if self._connector._share_cookies:
-                self._connector.update_cookies(resp.cookies)
+            self._cookie_jar.update_cookies(resp.cookies, resp.url_obj)
 
             # redirects
             if resp.status in (301, 302, 303, 307) and allow_redirects:
@@ -198,10 +198,19 @@ class ClientSession:
                 if max_redirects and redirects >= max_redirects:
                     resp.close()
                     break
+                else:
+                    # TODO: close the connection if BODY is large enough
+                    # Redirect with big BODY is forbidden by HTTP protocol
+                    # but malformed server may send illegal response.
+                    # Small BODIES with text like "Not Found" are still
+                    # perfectly fine and should be accepted.
+                    yield from resp.release()
 
                 # For 301 and 302, mimic IE behaviour, now changed in RFC.
                 # Details: https://github.com/kennethreitz/requests/pull/269
-                if resp.status != 307:
+                if (resp.status == 303 and resp.method != hdrs.METH_HEAD) \
+                   or (resp.status in (301, 302) and
+                       resp.method == hdrs.METH_POST):
                     method = hdrs.METH_GET
                     data = None
                     if headers.get(hdrs.CONTENT_LENGTH):
@@ -209,15 +218,22 @@ class ClientSession:
 
                 r_url = (resp.headers.get(hdrs.LOCATION) or
                          resp.headers.get(hdrs.URI))
+                if r_url is None:
+                    raise RuntimeError("{0.method} {0.url_obj} returns "
+                                       "a redirect [{0.status}] status "
+                                       "but response lacks a Location "
+                                       "or URI HTTP header".format(resp))
+                r_url = URL(r_url)
 
-                scheme = urllib.parse.urlsplit(r_url)[0]
+                scheme = r_url.scheme
                 if scheme not in ('http', 'https', ''):
                     resp.close()
                     raise ValueError('Can redirect only to http or https')
                 elif not scheme:
-                    r_url = urllib.parse.urljoin(url, r_url)
+                    r_url = url.join(r_url)
 
                 url = r_url
+                params = None
                 yield from resp.release()
                 continue
 
@@ -232,7 +248,10 @@ class ClientSession:
                    autoclose=True,
                    autoping=True,
                    auth=None,
-                   origin=None):
+                   origin=None,
+                   headers=None,
+                   proxy=None,
+                   proxy_auth=None):
         """Initiate websocket connection."""
         return _WSRequestContextManager(
             self._ws_connect(url,
@@ -241,7 +260,10 @@ class ClientSession:
                              autoclose=autoclose,
                              autoping=autoping,
                              auth=auth,
-                             origin=origin))
+                             origin=origin,
+                             headers=headers,
+                             proxy=proxy,
+                             proxy_auth=proxy_auth))
 
     @asyncio.coroutine
     def _ws_connect(self, url, *,
@@ -250,16 +272,27 @@ class ClientSession:
                     autoclose=True,
                     autoping=True,
                     auth=None,
-                    origin=None):
+                    origin=None,
+                    headers=None,
+                    proxy=None,
+                    proxy_auth=None):
 
         sec_key = base64.b64encode(os.urandom(16))
 
-        headers = {
+        if headers is None:
+            headers = CIMultiDict()
+
+        default_headers = {
             hdrs.UPGRADE: hdrs.WEBSOCKET,
             hdrs.CONNECTION: hdrs.UPGRADE,
             hdrs.SEC_WEBSOCKET_VERSION: '13',
             hdrs.SEC_WEBSOCKET_KEY: sec_key.decode(),
         }
+
+        for key, value in default_headers.items():
+            if key not in headers:
+                headers[key] = value
+
         if protocols:
             headers[hdrs.SEC_WEBSOCKET_PROTOCOL] = ','.join(protocols)
         if origin is not None:
@@ -268,7 +301,9 @@ class ClientSession:
         # send request
         resp = yield from self.get(url, headers=headers,
                                    read_until_eof=False,
-                                   auth=auth)
+                                   auth=auth,
+                                   proxy=proxy,
+                                   proxy_auth=proxy_auth)
 
         try:
             # check handshake
@@ -313,6 +348,7 @@ class ClientSession:
                         break
 
             reader = resp.connection.reader.set_parser(WebSocketParser)
+            resp.connection.writer.set_tcp_nodelay(True)
             writer = WebSocketWriter(resp.connection.writer, use_mask=True)
         except Exception:
             resp.close()
@@ -327,19 +363,6 @@ class ClientSession:
                                            autoping,
                                            self._loop)
 
-    def _update_cookies(self, cookies):
-        """Update shared cookies."""
-        if isinstance(cookies, dict):
-            cookies = cookies.items()
-
-        for name, value in cookies:
-            if isinstance(value, http.cookies.Morsel):
-                # use dict method because SimpleCookie class modifies value
-                # before Python 3.4
-                dict.__setitem__(self.cookies, name, value)
-            else:
-                self.cookies[name] = value
-
     def _prepare_headers(self, headers):
         """ Add default headers and transform it to CIMultiDict
         """
@@ -413,6 +436,9 @@ class ClientSession:
         if not self.closed:
             self._connector.close()
             self._connector = None
+        ret = helpers.create_future(self._loop)
+        ret.set_result(None)
+        return ret
 
     @property
     def closed(self):
@@ -428,9 +454,19 @@ class ClientSession:
         return self._connector
 
     @property
-    def cookies(self):
+    def cookie_jar(self):
         """The session cookies."""
-        return self._cookies
+        return self._cookie_jar
+
+    @property
+    def version(self):
+        """The session HTTP protocol version."""
+        return self._version
+
+    @property
+    def loop(self):
+        """Session's loop."""
+        return self._loop
 
     def detach(self):
         """Detach connector from session without closing the former.
@@ -440,11 +476,22 @@ class ClientSession:
         self._connector = None
 
     def __enter__(self):
+        warnings.warn("Use async with instead", DeprecationWarning)
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
+    if PY_35:
+        @asyncio.coroutine
+        def __aenter__(self):
+            return self
+
+        @asyncio.coroutine
+        def __aexit__(self, exc_type, exc_val, exc_tb):
+            yield from self.close()
+
+
 if PY_35:
     from collections.abc import Coroutine
     base = Coroutine
@@ -454,25 +501,14 @@ else:
 
 class _BaseRequestContextManager(base):
 
-    __slots__ = ('_coro', '_resp')
+    __slots__ = ('_coro', '_resp', 'send', 'throw', 'close')
 
     def __init__(self, coro):
         self._coro = coro
         self._resp = None
-
-    def send(self, value):
-        return self._coro.send(value)
-
-    def throw(self, typ, val=None, tb=None):
-        if val is None:
-            return self._coro.throw(typ)
-        elif tb is None:
-            return self._coro.throw(typ, val)
-        else:
-            return self._coro.throw(typ, val, tb)
-
-    def close(self):
-        return self._coro.close()
+        self.send = coro.send
+        self.throw = coro.throw
+        self.close = coro.close
 
     @property
     def gi_frame(self):
@@ -509,8 +545,8 @@ if not PY_35:
     try:
         from asyncio import coroutines
         coroutines._COROUTINE_TYPES += (_BaseRequestContextManager,)
-    except:
-        pass
+    except:  # pragma: no cover
+        pass  # Python 3.4.2 and 3.4.3 has no coroutines._COROUTINE_TYPES
 
 
 class _RequestContextManager(_BaseRequestContextManager):
@@ -543,7 +579,7 @@ class _DetachedRequestContextManager(_RequestContextManager):
         try:
             return (yield from self._coro)
         except:
-            self._session.close()
+            yield from self._session.close()
             raise
 
     if PY_35:
@@ -551,7 +587,7 @@ class _DetachedRequestContextManager(_RequestContextManager):
             try:
                 return (yield from self._coro)
             except:
-                self._session.close()
+                yield from self._session.close()
                 raise
 
     def __del__(self):
@@ -580,7 +616,7 @@ def request(method, url, *,
             allow_redirects=True,
             max_redirects=10,
             encoding='utf-8',
-            version=aiohttp.HttpVersion11,
+            version=None,
             compress=None,
             chunked=None,
             expect100=False,
@@ -588,37 +624,36 @@ def request(method, url, *,
             loop=None,
             read_until_eof=True,
             request_class=None,
-            response_class=None):
+            response_class=None,
+            proxy=None,
+            proxy_auth=None):
     """Constructs and sends a request. Returns response object.
 
-    :param str method: http method
-    :param str url: request url
-    :param params: (optional) Dictionary or bytes to be sent in the query
+    method - HTTP method
+    url - request url
+    params - (optional) Dictionary or bytes to be sent in the query
       string of the new request
-    :param data: (optional) Dictionary, bytes, or file-like object to
+    data - (optional) Dictionary, bytes, or file-like object to
       send in the body of the request
-    :param dict headers: (optional) Dictionary of HTTP Headers to send with
+    headers - (optional) Dictionary of HTTP Headers to send with
       the request
-    :param dict cookies: (optional) Dict object to send with the request
-    :param auth: (optional) BasicAuth named tuple represent HTTP Basic Auth
-    :type auth: aiohttp.helpers.BasicAuth
-    :param bool allow_redirects: (optional) If set to False, do not follow
+    cookies - (optional) Dict object to send with the request
+    auth - (optional) BasicAuth named tuple represent HTTP Basic Auth
+    auth - aiohttp.helpers.BasicAuth
+    allow_redirects - (optional) If set to False, do not follow
       redirects
-    :param version: Request http version.
-    :type version: aiohttp.protocol.HttpVersion
-    :param bool compress: Set to True if request has to be compressed
+    version - Request HTTP version.
+    compress - Set to True if request has to be compressed
        with deflate encoding.
-    :param chunked: Set to chunk size for chunked transfer encoding.
-    :type chunked: bool or int
-    :param bool expect100: Expect 100-continue response from server.
-    :param connector: BaseConnector sub-class instance to support
+    chunked - Set to chunk size for chunked transfer encoding.
+    expect100 - Expect 100-continue response from server.
+    connector - BaseConnector sub-class instance to support
        connection pooling.
-    :type connector: aiohttp.connector.BaseConnector
-    :param bool read_until_eof: Read response until eof if response
+    read_until_eof - Read response until eof if response
        does not have Content-Length header.
-    :param request_class: (optional) Custom Request class implementation.
-    :param response_class: (optional) Custom Response class implementation.
-    :param loop: Optional event loop.
+    request_class - (optional) Custom Request class implementation.
+    response_class - (optional) Custom Response class implementation.
+    loop - Optional event loop.
 
     Usage::
 
@@ -629,6 +664,7 @@ def request(method, url, *,
       >>> data = yield from resp.read()
 
     """
+    warnings.warn("Use ClientSession().request() instead", DeprecationWarning)
     if connector is None:
         connector = aiohttp.TCPConnector(loop=loop, force_close=True)
 
@@ -658,35 +694,44 @@ def request(method, url, *,
                          compress=compress,
                          chunked=chunked,
                          expect100=expect100,
-                         read_until_eof=read_until_eof),
+                         read_until_eof=read_until_eof,
+                         proxy=proxy,
+                         proxy_auth=proxy_auth,),
         session=session)
 
 
 def get(url, **kwargs):
+    warnings.warn("Use ClientSession().get() instead", DeprecationWarning)
     return request(hdrs.METH_GET, url, **kwargs)
 
 
 def options(url, **kwargs):
+    warnings.warn("Use ClientSession().options() instead", DeprecationWarning)
     return request(hdrs.METH_OPTIONS, url, **kwargs)
 
 
 def head(url, **kwargs):
+    warnings.warn("Use ClientSession().head() instead", DeprecationWarning)
     return request(hdrs.METH_HEAD, url, **kwargs)
 
 
 def post(url, **kwargs):
+    warnings.warn("Use ClientSession().post() instead", DeprecationWarning)
     return request(hdrs.METH_POST, url, **kwargs)
 
 
 def put(url, **kwargs):
+    warnings.warn("Use ClientSession().put() instead", DeprecationWarning)
     return request(hdrs.METH_PUT, url, **kwargs)
 
 
 def patch(url, **kwargs):
+    warnings.warn("Use ClientSession().patch() instead", DeprecationWarning)
     return request(hdrs.METH_PATCH, url, **kwargs)
 
 
 def delete(url, **kwargs):
+    warnings.warn("Use ClientSession().delete() instead", DeprecationWarning)
     return request(hdrs.METH_DELETE, url, **kwargs)
 
 
@@ -694,6 +739,8 @@ def ws_connect(url, *, protocols=(), timeout=10.0, connector=None, auth=None,
                ws_response_class=ClientWebSocketResponse, autoclose=True,
                autoping=True, loop=None, origin=None, headers=None):
 
+    warnings.warn("Use ClientSession().ws_connect() instead",
+                  DeprecationWarning)
     if loop is None:
         loop = asyncio.get_event_loop()
 
