@@ -1,5 +1,4 @@
 import asyncio
-import collections
 import http.cookies
 import io
 import json
@@ -7,31 +6,30 @@ import mimetypes
 import os
 import sys
 import traceback
-import urllib.parse
 import warnings
 
+from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy
+from yarl import URL
+
+import aiohttp
+
+from . import hdrs, helpers, streams
+from .helpers import HeadersMixin, Timeout
+from .log import client_logger
+from .multipart import MultipartWriter
+from .protocol import HttpMessage
+from .streams import FlowControlStreamReader
+
 try:
     import cchardet as chardet
-except ImportError:
+except ImportError:  # pragma: no cover
     import chardet
 
-import aiohttp
-from . import hdrs, helpers, streams
-from .log import client_logger
-from .streams import EOF_MARKER, FlowControlStreamReader
-from .multidict import (CIMultiDictProxy, MultiDictProxy, MultiDict,
-                        CIMultiDict)
-from .multipart import MultipartWriter
-from .protocol import HttpMessage
-
 
 __all__ = ('ClientRequest', 'ClientResponse')
 
 PY_35 = sys.version_info >= (3, 5)
 
-HTTP_PORT = 80
-HTTPS_PORT = 443
-
 
 class ClientRequest:
 
@@ -66,87 +64,74 @@ class ClientRequest:
                  auth=None, encoding='utf-8',
                  version=aiohttp.HttpVersion11, compress=None,
                  chunked=None, expect100=False,
-                 loop=None, response_class=None):
+                 loop=None, response_class=None,
+                 proxy=None, proxy_auth=None,
+                 timeout=5*60):
 
         if loop is None:
             loop = asyncio.get_event_loop()
 
-        self.url = url
+        assert isinstance(url, URL), url
+        assert isinstance(proxy, (URL, type(None))), proxy
+
+        if params:
+            q = MultiDict(url.query)
+            url2 = url.with_query(params)
+            q.extend(url2.query)
+            url = url.with_query(q)
+        self.url = url.with_fragment(None)
         self.method = method.upper()
         self.encoding = encoding
         self.chunked = chunked
         self.compress = compress
         self.loop = loop
         self.response_class = response_class or ClientResponse
+        self._timeout = timeout
 
         if loop.get_debug():
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
         self.update_version(version)
         self.update_host(url)
-        self.update_path(params)
         self.update_headers(headers)
         self.update_auto_headers(skip_auto_headers)
         self.update_cookies(cookies)
-        self.update_content_encoding()
+        self.update_content_encoding(data)
         self.update_auth(auth)
+        self.update_proxy(proxy, proxy_auth)
 
         self.update_body_from_data(data, skip_auto_headers)
         self.update_transfer_encoding()
         self.update_expect_continue(expect100)
 
+    @property
+    def host(self):
+        return self.url.host
+
+    @property
+    def port(self):
+        return self.url.port
+
     def update_host(self, url):
         """Update destination host, port and connection type (ssl)."""
-        url_parsed = urllib.parse.urlsplit(url)
-
-        # check for network location part
-        netloc = url_parsed.netloc
-        if not netloc:
-            raise ValueError('Host could not be detected.')
-
         # get host/port
-        host = url_parsed.hostname
-        if not host:
+        if not url.host:
             raise ValueError('Host could not be detected.')
 
-        try:
-            port = url_parsed.port
-        except ValueError:
-            raise ValueError(
-                'Port number could not be converted.') from None
-
-        # check domain idna encoding
-        try:
-            netloc = netloc.encode('idna').decode('utf-8')
-            host = host.encode('idna').decode('utf-8')
-        except UnicodeError:
-            raise ValueError('URL has an invalid label.')
-
         # basic auth info
-        username, password = url_parsed.username, url_parsed.password
+        username, password = url.user, url.password
         if username:
             self.auth = helpers.BasicAuth(username, password or '')
-            netloc = netloc.split('@', 1)[1]
 
         # Record entire netloc for usage in host header
-        self.netloc = netloc
 
-        scheme = url_parsed.scheme
+        scheme = url.scheme
         self.ssl = scheme in ('https', 'wss')
 
-        # set port number if it isn't already set
-        if not port:
-            if self.ssl:
-                port = HTTPS_PORT
-            else:
-                port = HTTP_PORT
-
-        self.host, self.port, self.scheme = host, port, scheme
-
     def update_version(self, version):
         """Convert request version to two elements tuple.
 
-        parser http version '1.1' => (1, 1)
+        parser HTTP version '1.1' => (1, 1)
         """
         if isinstance(version, str):
             v = [l.strip() for l in version.split('.', 1)]
@@ -158,29 +143,6 @@ class ClientRequest:
                     .format(version)) from None
         self.version = version
 
-    def update_path(self, params):
-        """Build path."""
-        # extract path
-        scheme, netloc, path, query, fragment = urllib.parse.urlsplit(self.url)
-        if not path:
-            path = '/'
-
-        if isinstance(params, collections.Mapping):
-            params = list(params.items())
-
-        if params:
-            if not isinstance(params, str):
-                params = urllib.parse.urlencode(params)
-            if query:
-                query = '%s&%s' % (query, params)
-            else:
-                query = params
-
-        self.path = urllib.parse.urlunsplit(('', '', helpers.requote_uri(path),
-                                             query, fragment))
-        self.url = urllib.parse.urlunsplit(
-            (scheme, netloc, self.path, '', ''))
-
     def update_headers(self, headers):
         """Update request headers."""
         self.headers = CIMultiDict()
@@ -203,7 +165,10 @@ class ClientRequest:
 
         # add host
         if hdrs.HOST not in used_headers:
-            self.headers[hdrs.HOST] = self.netloc
+            netloc = self.url.raw_host
+            if not self.url.is_default_port():
+                netloc += ':' + str(self.url.port)
+            self.headers[hdrs.HOST] = netloc
 
         if hdrs.USER_AGENT not in used_headers:
             self.headers[hdrs.USER_AGENT] = self.SERVER_SOFTWARE
@@ -218,19 +183,22 @@ class ClientRequest:
             c.load(self.headers.get(hdrs.COOKIE, ''))
             del self.headers[hdrs.COOKIE]
 
-        if isinstance(cookies, dict):
-            cookies = cookies.items()
-
-        for name, value in cookies:
+        for name, value in cookies.items():
             if isinstance(value, http.cookies.Morsel):
-                c[value.key] = value.value
+                # Preserve coded_value
+                mrsl_val = value.get(value.key, http.cookies.Morsel())
+                mrsl_val.set(value.key, value.value, value.coded_value)
+                c[name] = mrsl_val
             else:
                 c[name] = value
 
         self.headers[hdrs.COOKIE] = c.output(header='', sep=';').strip()
 
-    def update_content_encoding(self):
+    def update_content_encoding(self, data):
         """Set request content encoding."""
+        if not data:
+            return
+
         enc = self.headers.get(hdrs.CONTENT_ENCODING, '').lower()
         if enc:
             if self.compress is not False:
@@ -251,9 +219,7 @@ class ClientRequest:
             return
 
         if not isinstance(auth, helpers.BasicAuth):
-            warnings.warn(
-                'BasicAuth() tuple is required instead ', DeprecationWarning)
-            auth = helpers.BasicAuth(*auth)
+            raise TypeError('BasicAuth() tuple is required instead')
 
         self.headers[hdrs.AUTHORIZATION] = auth.encode()
 
@@ -272,7 +238,8 @@ class ClientRequest:
             if hdrs.CONTENT_LENGTH not in self.headers and not self.chunked:
                 self.headers[hdrs.CONTENT_LENGTH] = str(len(self.body))
 
-        elif isinstance(data, (asyncio.StreamReader, streams.DataQueue)):
+        elif isinstance(data, (asyncio.StreamReader, streams.StreamReader,
+                               streams.DataQueue)):
             self.body = data
 
         elif asyncio.iscoroutine(data):
@@ -290,7 +257,8 @@ class ClientRequest:
                 size = len(data.getbuffer())
                 self.headers[hdrs.CONTENT_LENGTH] = str(size)
                 self.chunked = False
-            elif not self.chunked and isinstance(data, io.BufferedReader):
+            elif (not self.chunked and
+                  isinstance(data, (io.BufferedReader, io.BufferedRandom))):
                 # Not chunking if content-length can be determined
                 try:
                     size = os.fstat(data.fileno()).st_size - data.tell()
@@ -362,7 +330,15 @@ class ClientRequest:
             expect = True
 
         if expect:
-            self._continue = asyncio.Future(loop=self.loop)
+            self._continue = helpers.create_future(self.loop)
+
+    def update_proxy(self, proxy, proxy_auth):
+        if proxy and not proxy.scheme == 'http':
+            raise ValueError("Only http proxies are supported")
+        if proxy_auth and not isinstance(proxy_auth, helpers.BasicAuth):
+            raise ValueError("proxy_auth must be None or BasicAuth() tuple")
+        self.proxy = proxy
+        self.proxy_auth = proxy_auth
 
     @asyncio.coroutine
     def write_bytes(self, request, reader):
@@ -388,7 +364,7 @@ class ClientRequest:
                             yield from request.write(exc.value, drain=True)
                         break
                     except:
-                        self.response.close(True)
+                        self.response.close()
                         raise
 
                     if isinstance(result, asyncio.Future):
@@ -406,7 +382,8 @@ class ClientRequest:
                             'Bytes object is expected, got: %s.' %
                             type(result))
 
-            elif isinstance(self.body, asyncio.StreamReader):
+            elif isinstance(self.body, (asyncio.StreamReader,
+                                        streams.StreamReader)):
                 chunk = yield from self.body.read(streams.DEFAULT_LIMIT)
                 while chunk:
                     yield from request.write(chunk, drain=True)
@@ -416,7 +393,7 @@ class ClientRequest:
                 while True:
                     try:
                         chunk = yield from self.body.read()
-                        if chunk is EOF_MARKER:
+                        if not chunk:
                             break
                         yield from request.write(chunk, drain=True)
                     except streams.EofStream:
@@ -427,13 +404,13 @@ class ClientRequest:
                 while chunk:
                     request.write(chunk)
                     chunk = self.body.read(self.chunked)
-
             else:
                 if isinstance(self.body, (bytes, bytearray)):
                     self.body = (self.body,)
 
                 for chunk in self.body:
                     request.write(chunk)
+
         except Exception as exc:
             new_exc = aiohttp.ClientRequestError(
                 'Can not write request body for %s' % self.url)
@@ -458,7 +435,21 @@ class ClientRequest:
         self._writer = None
 
     def send(self, writer, reader):
-        request = aiohttp.Request(writer, self.method, self.path, self.version)
+        # Specify request target:
+        # - CONNECT request must send authority form URI
+        # - not CONNECT proxy must send absolute form URI
+        # - most common is origin form URI
+        if self.method == hdrs.METH_CONNECT:
+            path = '{}:{}'.format(self.url.raw_host, self.url.port)
+        elif self.proxy and not self.ssl:
+            path = str(self.url)
+        else:
+            path = self.url.raw_path
+            if self.url.raw_query_string:
+                path += '?' + self.url.raw_query_string
+
+        request = aiohttp.Request(writer, self.method, path,
+                                  self.version)
 
         if self.compress:
             request.add_compression_filter(self.compress)
@@ -481,8 +472,9 @@ class ClientRequest:
             self.write_bytes(request, reader), loop=self.loop)
 
         self.response = self.response_class(
-            self.method, self.url, self.host,
-            writer=self._writer, continue100=self._continue)
+            self.method, self.url,
+            writer=self._writer, continue100=self._continue,
+            timeout=self._timeout)
         self.response._post_init(self.loop)
         return self.response
 
@@ -496,24 +488,21 @@ class ClientRequest:
 
     def terminate(self):
         if self._writer is not None:
-            if hasattr(self.loop, 'is_closed'):
-                if not self.loop.is_closed():
-                    self._writer.cancel()
-            else:
+            if not self.loop.is_closed():
                 self._writer.cancel()
             self._writer = None
 
 
-class ClientResponse:
+class ClientResponse(HeadersMixin):
 
     # from the Status-Line of the response
     version = None  # HTTP-Version
     status = None   # Status-Code
     reason = None   # Reason-Phrase
 
-    cookies = None  # Response cookies (Set-Cookie)
     content = None  # Payload stream
     headers = None  # Response headers, CIMultiDictProxy
+    raw_headers = None  # Response raw headers, a sequence of pairs
 
     _connection = None  # current connection
     flow_control_class = FlowControlStreamReader  # reader flow control
@@ -525,18 +514,38 @@ class ClientResponse:
     _loop = None
     _closed = True  # to allow __del__ for non-initialized properly response
 
-    def __init__(self, method, url, host='', *, writer=None, continue100=None):
-        super().__init__()
+    def __init__(self, method, url, *, writer=None, continue100=None,
+                 timeout=5*60):
+        assert isinstance(url, URL)
 
         self.method = method
-        self.url = url
-        self.host = host
+        self._url_obj = url
         self._content = None
         self._writer = writer
         self._continue = continue100
         self._closed = False
         self._should_close = True  # override by message.should_close later
         self._history = ()
+        self._timeout = timeout
+        self.cookies = http.cookies.SimpleCookie()
+
+    @property
+    def url_obj(self):
+        return self._url_obj
+
+    @property
+    def url(self):
+        warnings.warn("Deprecated, use .url_obj",
+                      DeprecationWarning,
+                      stacklevel=2)
+        return str(self._url_obj)
+
+    @property
+    def host(self):
+        warnings.warn("Deprecated, use .url_obj.host",
+                      DeprecationWarning,
+                      stacklevel=2)
+        return self._url_obj.host
 
     def _post_init(self, loop):
         self._loop = loop
@@ -544,6 +553,8 @@ class ClientResponse:
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
     def __del__(self, _warnings=warnings):
+        if self._loop is None:
+            return  # not started
         if self._closed:
             return
         self.close()
@@ -558,8 +569,16 @@ class ClientResponse:
 
     def __repr__(self):
         out = io.StringIO()
+        ascii_encodable_url = str(self.url)
+        if self.reason:
+            ascii_encodable_reason = self.reason.encode('ascii',
+                                                        'backslashreplace') \
+                .decode('ascii')
+        else:
+            ascii_encodable_reason = self.reason
         print('<ClientResponse({}) [{} {}]>'.format(
-            self.url, self.status, self.reason), file=out)
+            ascii_encodable_url, self.status, ascii_encodable_reason),
+            file=out)
         print(self.headers, file=out)
         return out.getvalue()
 
@@ -569,17 +588,14 @@ class ClientResponse:
 
     @property
     def history(self):
-        """A sequence of of responses, if redirects occured."""
+        """A sequence of of responses, if redirects occurred."""
         return self._history
 
-    def waiting_for_continue(self):
-        return self._continue is not None
-
     def _setup_connection(self, connection):
         self._reader = connection.reader
         self._connection = connection
         self.content = self.flow_control_class(
-            connection.reader, loop=connection.loop)
+            connection.reader, loop=connection.loop, timeout=self._timeout)
 
     def _need_parse_response_body(self):
         return (self.method.lower() != 'head' and
@@ -594,7 +610,8 @@ class ClientResponse:
             httpstream = self._reader.set_parser(self._response_parser)
 
             # read response
-            message = yield from httpstream.read()
+            with Timeout(self._timeout, loop=self._loop):
+                message = yield from httpstream.read()
             if message.code != 100:
                 break
 
@@ -610,52 +627,53 @@ class ClientResponse:
 
         # headers
         self.headers = CIMultiDictProxy(message.headers)
+        self.raw_headers = tuple(message.raw_headers)
 
         # payload
-        response_with_body = self._need_parse_response_body()
+        rwb = self._need_parse_response_body()
         self._reader.set_parser(
             aiohttp.HttpPayloadParser(message,
                                       readall=read_until_eof,
-                                      response_with_body=response_with_body),
+                                      response_with_body=rwb),
             self.content)
 
         # cookies
-        self.cookies = http.cookies.SimpleCookie()
-        if hdrs.SET_COOKIE in self.headers:
-            for hdr in self.headers.getall(hdrs.SET_COOKIE):
-                try:
-                    self.cookies.load(hdr)
-                except http.cookies.CookieError as exc:
-                    client_logger.warning(
-                        'Can not load response cookies: %s', exc)
+        for hdr in self.headers.getall(hdrs.SET_COOKIE, ()):
+            try:
+                self.cookies.load(hdr)
+            except http.cookies.CookieError as exc:
+                client_logger.warning(
+                    'Can not load response cookies: %s', exc)
         return self
 
-    def close(self, force=True):
-        if not force:
-            warnings.warn("force parameter should be True", DeprecationWarning,
-                          stacklevel=2)
+    def close(self):
         if self._closed:
             return
 
         self._closed = True
 
-        if hasattr(self._loop, 'is_closed'):
-            if self._loop.is_closed():
-                return
+        if self._loop is None or self._loop.is_closed():
+            return
 
         if self._connection is not None:
             self._connection.close()
             self._connection = None
         self._cleanup_writer()
+        self._notify_content()
 
     @asyncio.coroutine
     def release(self):
+        if self._closed:
+            return
         try:
             content = self.content
-            if content is not None and not content.at_eof():
-                chunk = yield from content.readany()
-                while chunk is not EOF_MARKER or chunk:
-                    chunk = yield from content.readany()
+            if content is not None:
+                while not content.at_eof():
+                    yield from content.readany()
+        except Exception:
+            self._connection.close()
+            self._connection = None
+            raise
         finally:
             self._closed = True
             if self._connection is not None:
@@ -664,12 +682,25 @@ class ClientResponse:
                     self._reader.unset_parser()
                 self._connection = None
             self._cleanup_writer()
+            self._notify_content()
+
+    def raise_for_status(self):
+        if 400 <= self.status:
+            raise aiohttp.HttpProcessingError(
+                code=self.status,
+                message=self.reason)
 
     def _cleanup_writer(self):
         if self._writer is not None and not self._writer.done():
             self._writer.cancel()
         self._writer = None
 
+    def _notify_content(self):
+        content = self.content
+        if content and content.exception() is None and not content.is_eof():
+            content.set_exception(
+                aiohttp.ClientDisconnectedError('Connection closed'))
+
     @asyncio.coroutine
     def wait_for_close(self):
         if self._writer is not None:
@@ -680,7 +711,7 @@ class ClientResponse:
         yield from self.release()
 
     @asyncio.coroutine
-    def read(self, decode=False):
+    def read(self):
         """Read response payload."""
         if self._content is None:
             try:
@@ -691,15 +722,7 @@ class ClientResponse:
             else:
                 yield from self.release()
 
-        data = self._content
-
-        if decode:
-            warnings.warn(
-                '.read(True) is deprecated. use .json() instead',
-                DeprecationWarning)
-            return (yield from self.json())
-
-        return data
+        return self._content
 
     def _get_encoding(self):
         ctype = self.headers.get(hdrs.CONTENT_TYPE, '').lower()
@@ -707,7 +730,11 @@ class ClientResponse:
 
         encoding = params.get('charset')
         if not encoding:
-            encoding = chardet.detect(self._content)['encoding']
+            if mtype == 'application' and stype == 'json':
+                # RFC 7159 states that the default encoding is UTF-8.
+                encoding = 'utf-8'
+            else:
+                encoding = chardet.detect(self._content)['encoding']
         if not encoding:
             encoding = 'utf-8'
 
