@@ -1,27 +1,27 @@
 import asyncio
-import aiohttp
 import functools
 import http.cookies
 import ssl
-import socket
 import sys
 import traceback
 import warnings
-
 from collections import defaultdict
 from hashlib import md5, sha1, sha256
 from itertools import chain
 from math import ceil
 from types import MappingProxyType
 
-from . import hdrs
-from .client import ClientRequest
-from .errors import ServerDisconnectedError
-from .errors import HttpProxyError, ProxyConnectionError
-from .errors import ClientOSError, ClientTimeoutError
-from .errors import FingerprintMismatch
-from .helpers import BasicAuth
+from yarl import URL
 
+import aiohttp
+
+from . import hdrs, helpers
+from .client import ClientRequest
+from .errors import (ClientOSError, ClientTimeoutError, FingerprintMismatch,
+                     HttpProxyError, ProxyConnectionError,
+                     ServerDisconnectedError)
+from .helpers import is_ip_address, sentinel
+from .resolver import DefaultResolver
 
 __all__ = ('BaseConnector', 'TCPConnector', 'ProxyConnector', 'UnixConnector')
 
@@ -34,7 +34,7 @@ HASHFUNC_BY_DIGESTLEN = {
 }
 
 
-class Connection(object):
+class Connection:
 
     _source_traceback = None
     _transport = None
@@ -59,9 +59,8 @@ class Connection(object):
         if self._transport is not None:
             _warnings.warn('Unclosed connection {!r}'.format(self),
                            ResourceWarning)
-            if hasattr(self._loop, 'is_closed'):
-                if self._loop.is_closed():
-                    return
+            if self._loop.is_closed():
+                return
 
             self._connector._release(
                 self._key, self._request, self._transport, self._protocol,
@@ -92,6 +91,8 @@ class Connection(object):
             self._transport = None
 
     def detach(self):
+        if self._transport is not None:
+            self._connector._release_acquired(self._key, self._transport)
         self._transport = None
 
     @property
@@ -102,19 +103,30 @@ class Connection(object):
 class BaseConnector(object):
     """Base connector class.
 
-    :param conn_timeout: (optional) Connect timeout.
-    :param keepalive_timeout: (optional) Keep-alive timeout.
-    :param bool force_close: Set to True to force close and do reconnect
+    conn_timeout - (optional) Connect timeout.
+    keepalive_timeout - (optional) Keep-alive timeout.
+    force_close - Set to True to force close and do reconnect
         after each request (and between redirects).
-    :param loop: Optional event loop.
+    limit - The limit of simultaneous connections to the same endpoint.
+    loop - Optional event loop.
     """
 
     _closed = True  # prevent AttributeError in __del__ if ctor was failed
     _source_traceback = None
 
-    def __init__(self, *, conn_timeout=None, keepalive_timeout=30,
-                 share_cookies=False, force_close=False, limit=None,
+    def __init__(self, *, conn_timeout=None, keepalive_timeout=sentinel,
+                 force_close=False, limit=20,
                  loop=None):
+
+        if force_close:
+            if keepalive_timeout is not None and \
+               keepalive_timeout is not sentinel:
+                raise ValueError('keepalive_timeout cannot '
+                                 'be set if force_close is True')
+        else:
+            if keepalive_timeout is sentinel:
+                keepalive_timeout = 30
+
         if loop is None:
             loop = asyncio.get_event_loop()
 
@@ -126,11 +138,6 @@ class BaseConnector(object):
         self._acquired = defaultdict(set)
         self._conn_timeout = conn_timeout
         self._keepalive_timeout = keepalive_timeout
-        if share_cookies:
-            warnings.warn(
-                'Using `share_cookies` is deprecated. '
-                'Use Session object instead', DeprecationWarning)
-        self._share_cookies = share_cookies
         self._cleanup_handle = None
         self._force_close = force_close
         self._limit = limit
@@ -162,6 +169,12 @@ class BaseConnector(object):
             context['source_traceback'] = self._source_traceback
         self._loop.call_exception_handler(context)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *exc):
+        self.close()
+
     @property
     def force_close(self):
         """Ultimately close connection on releasing if True."""
@@ -174,7 +187,8 @@ class BaseConnector(object):
         Endpoints are the same if they are have equal
         (host, port, is_ssl) triple.
 
-        If limit is None the connector has no limit (default).
+        If limit is None the connector has no limit.
+        The default limit size is 20.
         """
         return self._limit
 
@@ -222,14 +236,15 @@ class BaseConnector(object):
 
     def close(self):
         """Close all opened transports."""
+        ret = helpers.create_future(self._loop)
+        ret.set_result(None)
         if self._closed:
-            return
+            return ret
         self._closed = True
 
         try:
-            if hasattr(self._loop, 'is_closed'):
-                if self._loop.is_closed():
-                    return
+            if self._loop.is_closed():
+                return ret
 
             for key, data in self._conns.items():
                 for transport, proto, t0 in data:
@@ -245,6 +260,7 @@ class BaseConnector(object):
             self._conns.clear()
             self._acquired.clear()
             self._cleanup_handle = None
+        return ret
 
     @property
     def closed(self):
@@ -254,24 +270,6 @@ class BaseConnector(object):
         """
         return self._closed
 
-    def update_cookies(self, cookies):
-        """Update shared cookies.
-
-        Deprecated, use ClientSession instead.
-        """
-        if isinstance(cookies, dict):
-            cookies = cookies.items()
-
-        for name, value in cookies:
-            if PY_343:
-                self.cookies[name] = value
-            else:
-                if isinstance(value, http.cookies.Morsel):
-                    # use dict method because SimpleCookie class modifies value
-                    dict.__setitem__(self.cookies, name, value)
-                else:
-                    self.cookies[name] = value
-
     @asyncio.coroutine
     def connect(self, req):
         """Get from pool or create new connection."""
@@ -279,7 +277,7 @@ class BaseConnector(object):
 
         limit = self._limit
         if limit is not None:
-            fut = asyncio.Future(loop=self._loop)
+            fut = helpers.create_future(self._loop)
             waiters = self._waiters[key]
 
             # The limit defines the maximum number of concurrent connections
@@ -294,27 +292,33 @@ class BaseConnector(object):
             # This connection will now count towards the limit.
             waiters.append(fut)
 
-            yield from fut
+        try:
+            if limit is not None:
+                yield from fut
 
-        transport, proto = self._get(key)
-        if transport is None:
-            try:
-                if self._conn_timeout:
-                    transport, proto = yield from asyncio.wait_for(
-                        self._create_connection(req),
-                        self._conn_timeout, loop=self._loop)
-                else:
-                    transport, proto = yield from self._create_connection(req)
+            transport, proto = self._get(key)
+            if transport is None:
+                try:
+                    if self._conn_timeout:
+                        transport, proto = yield from asyncio.wait_for(
+                            self._create_connection(req),
+                            self._conn_timeout, loop=self._loop)
+                    else:
+                        transport, proto = \
+                            yield from self._create_connection(req)
 
-            except asyncio.TimeoutError as exc:
-                raise ClientTimeoutError(
-                    'Connection timeout to host {0[0]}:{0[1]} ssl:{0[2]}'
-                    .format(key)) from exc
-            except OSError as exc:
-                raise ClientOSError(
-                    exc.errno,
-                    'Cannot connect to host {0[0]}:{0[1]} ssl:{0[2]} [{1}]'
-                    .format(key, exc.strerror)) from exc
+                except asyncio.TimeoutError as exc:
+                    raise ClientTimeoutError(
+                        'Connection timeout to host {0[0]}:{0[1]} ssl:{0[2]}'
+                        .format(key)) from exc
+                except OSError as exc:
+                    raise ClientOSError(
+                        exc.errno,
+                        'Cannot connect to host {0[0]}:{0[1]} ssl:{0[2]} [{1}]'
+                        .format(key, exc.strerror)) from exc
+        except:
+            self._release_waiter(key)
+            raise
 
         self._acquired[key].add(transport)
         conn = Connection(self, key, req, transport, proto, self._loop)
@@ -341,7 +345,15 @@ class BaseConnector(object):
         del self._conns[key]
         return None, None
 
-    def _release(self, key, req, transport, protocol, *, should_close=False):
+    def _release_waiter(self, key):
+        waiters = self._waiters[key]
+        while waiters:
+            waiter = waiters.pop(0)
+            if not waiter.done():
+                waiter.set_result(None)
+                break
+
+    def _release_acquired(self, key, transport):
         if self._closed:
             # acquired connection is already released on connector closing
             return
@@ -352,15 +364,20 @@ class BaseConnector(object):
         except KeyError:  # pragma: no cover
             # this may be result of undetermenistic order of objects
             # finalization due garbage collection.
-            pass
-        else:
-            if self._limit is not None and len(acquired) < self._limit:
-                waiters = self._waiters[key]
-                while waiters:
-                    waiter = waiters.pop(0)
-                    if not waiter.done():
-                        waiter.set_result(None)
-                        break
+            return None
+
+        return acquired
+
+    def _release(self, key, req, transport, protocol, *, should_close=False):
+        if self._closed:
+            # acquired connection is already released on connector closing
+            return
+
+        acquired = self._release_acquired(key, transport)
+
+        if self._limit is not None and acquired is not None:
+            if len(acquired) < self._limit:
+                self._release_waiter(key)
 
         resp = req.response
 
@@ -389,28 +406,40 @@ class BaseConnector(object):
 
 _SSL_OP_NO_COMPRESSION = getattr(ssl, "OP_NO_COMPRESSION", 0)
 
-_marker = object()
-
 
 class TCPConnector(BaseConnector):
     """TCP connector.
 
-    :param bool verify_ssl: Set to True to check ssl certifications.
-    :param bytes fingerprint: Pass the binary md5, sha1, or sha256
+    verify_ssl - Set to True to check ssl certifications.
+    fingerprint - Pass the binary md5, sha1, or sha256
         digest of the expected certificate in DER format to verify
         that the certificate the server presents matches. See also
         https://en.wikipedia.org/wiki/Transport_Layer_Security#Certificate_pinning
-    :param bool resolve: Set to True to do DNS lookup for host name.
-    :param family: socket address family
-    :param args: see :class:`BaseConnector`
-    :param kwargs: see :class:`BaseConnector`
+    resolve - (Deprecated) Set to True to do DNS lookup for
+        host name.
+    resolver - Enable DNS lookups and use this
+        resolver
+    use_dns_cache - Use memory cache for DNS lookups.
+    family - socket address family
+    local_addr - local tuple of (host, port) to bind socket to
+
+    conn_timeout - (optional) Connect timeout.
+    keepalive_timeout - (optional) Keep-alive timeout.
+    force_close - Set to True to force close and do reconnect
+        after each request (and between redirects).
+    limit - The limit of simultaneous connections to the same endpoint.
+    loop - Optional event loop.
     """
 
     def __init__(self, *, verify_ssl=True, fingerprint=None,
-                 resolve=_marker, use_dns_cache=_marker,
-                 family=0, ssl_context=None,
-                 **kwargs):
-        super().__init__(**kwargs)
+                 resolve=sentinel, use_dns_cache=sentinel,
+                 family=0, ssl_context=None, local_addr=None, resolver=None,
+                 conn_timeout=None, keepalive_timeout=sentinel,
+                 force_close=False, limit=20,
+                 loop=None):
+        super().__init__(conn_timeout=conn_timeout,
+                         keepalive_timeout=keepalive_timeout,
+                         force_close=force_close, limit=limit, loop=loop)
 
         if not verify_ssl and ssl_context is not None:
             raise ValueError(
@@ -424,29 +453,39 @@ class TCPConnector(BaseConnector):
             hashfunc = HASHFUNC_BY_DIGESTLEN.get(digestlen)
             if not hashfunc:
                 raise ValueError('fingerprint has invalid length')
+            elif hashfunc is md5 or hashfunc is sha1:
+                warnings.simplefilter('always')
+                warnings.warn('md5 and sha1 are insecure and deprecated. '
+                              'Use sha256.',
+                              DeprecationWarning, stacklevel=2)
             self._hashfunc = hashfunc
         self._fingerprint = fingerprint
 
-        if resolve is not _marker:
+        if resolve is not sentinel:
             warnings.warn(("resolve parameter is deprecated, "
                            "use use_dns_cache instead"),
                           DeprecationWarning, stacklevel=2)
 
-        if use_dns_cache is not _marker and resolve is not _marker:
+        if use_dns_cache is not sentinel and resolve is not sentinel:
             if use_dns_cache != resolve:
                 raise ValueError("use_dns_cache must agree with resolve")
             _use_dns_cache = use_dns_cache
-        elif use_dns_cache is not _marker:
+        elif use_dns_cache is not sentinel:
             _use_dns_cache = use_dns_cache
-        elif resolve is not _marker:
+        elif resolve is not sentinel:
             _use_dns_cache = resolve
         else:
-            _use_dns_cache = False
+            _use_dns_cache = True
+
+        if resolver is None:
+            resolver = DefaultResolver(loop=self._loop)
+        self._resolver = resolver
 
         self._use_dns_cache = _use_dns_cache
         self._cached_hosts = {}
         self._ssl_context = ssl_context
         self._family = family
+        self._local_addr = local_addr
 
     @property
     def verify_ssl(self):
@@ -529,26 +568,22 @@ class TCPConnector(BaseConnector):
 
     @asyncio.coroutine
     def _resolve_host(self, host, port):
+        if is_ip_address(host):
+            return [{'hostname': host, 'host': host, 'port': port,
+                     'family': self._family, 'proto': 0, 'flags': 0}]
+
         if self._use_dns_cache:
             key = (host, port)
 
             if key not in self._cached_hosts:
-                infos = yield from self._loop.getaddrinfo(
-                    host, port, type=socket.SOCK_STREAM, family=self._family)
+                self._cached_hosts[key] = yield from \
+                    self._resolver.resolve(host, port, family=self._family)
 
-                hosts = []
-                for family, _, proto, _, address in infos:
-                    hosts.append(
-                        {'hostname': host,
-                         'host': address[0], 'port': address[1],
-                         'family': family, 'proto': proto,
-                         'flags': socket.AI_NUMERICHOST})
-                self._cached_hosts[key] = hosts
-
-            return list(self._cached_hosts[key])
+            return self._cached_hosts[key]
         else:
-            return [{'hostname': host, 'host': host, 'port': port,
-                     'family': self._family, 'proto': 0, 'flags': 0}]
+            res = yield from self._resolver.resolve(
+                host, port, family=self._family)
+            return res
 
     @asyncio.coroutine
     def _create_connection(self, req):
@@ -556,6 +591,15 @@ class TCPConnector(BaseConnector):
 
         Has same keyword arguments as BaseEventLoop.create_connection.
         """
+        if req.proxy:
+            transport, proto = yield from self._create_proxy_connection(req)
+        else:
+            transport, proto = yield from self._create_direct_connection(req)
+
+        return transport, proto
+
+    @asyncio.coroutine
+    def _create_direct_connection(self, req):
         if req.ssl:
             sslcontext = self.ssl_context
         else:
@@ -572,7 +616,8 @@ class TCPConnector(BaseConnector):
                     self._factory, host, port,
                     ssl=sslcontext, family=hinfo['family'],
                     proto=hinfo['proto'], flags=hinfo['flags'],
-                    server_hostname=hinfo['hostname'] if sslcontext else None)
+                    server_hostname=hinfo['hostname'] if sslcontext else None,
+                    local_addr=self._local_addr)
                 has_cert = transp.get_extra_info('sslcontext')
                 if has_cert and self._fingerprint:
                     sock = transp.get_extra_info('socket')
@@ -597,63 +642,20 @@ class TCPConnector(BaseConnector):
                                 'Can not connect to %s:%s [%s]' %
                                 (req.host, req.port, exc.strerror)) from exc
 
-
-class ProxyConnector(TCPConnector):
-    """Http Proxy connector.
-
-    :param str proxy: Proxy URL address. Only http proxy supported.
-    :param proxy_auth: (optional) Proxy HTTP Basic Auth
-    :type proxy_auth: aiohttp.helpers.BasicAuth
-    :param args: see :class:`TCPConnector`
-    :param kwargs: see :class:`TCPConnector`
-
-    Usage:
-
-    >>> conn = ProxyConnector(proxy="http://some.proxy.com")
-    >>> session = ClientSession(connector=conn)
-    >>> resp = yield from session.get('http://python.org')
-
-    """
-
-    def __init__(self, proxy, *, proxy_auth=None, force_close=True,
-                 **kwargs):
-        super().__init__(force_close=force_close, **kwargs)
-        self._proxy = proxy
-        self._proxy_auth = proxy_auth
-        assert proxy.startswith('http://'), (
-            "Only http proxy supported", proxy)
-        assert proxy_auth is None or isinstance(proxy_auth, BasicAuth), (
-            "proxy_auth must be None or BasicAuth() tuple", proxy_auth)
-
-    @property
-    def proxy(self):
-        """Proxy URL."""
-        return self._proxy
-
-    @property
-    def proxy_auth(self):
-        """Proxy auth info.
-
-        Should be BasicAuth instance.
-        """
-        return self._proxy_auth
-
     @asyncio.coroutine
-    def _create_connection(self, req):
+    def _create_proxy_connection(self, req):
         proxy_req = ClientRequest(
-            hdrs.METH_GET, self._proxy,
-            headers={hdrs.HOST: req.host},
-            auth=self._proxy_auth,
+            hdrs.METH_GET, req.proxy,
+            headers={hdrs.HOST: req.headers[hdrs.HOST]},
+            auth=req.proxy_auth,
             loop=self._loop)
         try:
-            transport, proto = yield from super()._create_connection(proxy_req)
+            # create connection to proxy server
+            transport, proto = yield from self._create_direct_connection(
+                proxy_req)
         except OSError as exc:
             raise ProxyConnectionError(*exc.args) from exc
 
-        if not req.ssl:
-            req.path = '{scheme}://{host}{path}'.format(scheme=req.scheme,
-                                                        host=req.netloc,
-                                                        path=req.path)
         if hdrs.AUTHORIZATION in proxy_req.headers:
             auth = proxy_req.headers[hdrs.AUTHORIZATION]
             del proxy_req.headers[hdrs.AUTHORIZATION]
@@ -673,7 +675,7 @@ class ProxyConnector(TCPConnector):
             # to do this we must wrap raw socket into secure one
             # asyncio handles this perfectly
             proxy_req.method = hdrs.METH_CONNECT
-            proxy_req.path = '{}:{}'.format(req.host, req.port)
+            proxy_req.url = req.url
             key = (req.host, req.port, req.ssl)
             conn = Connection(self, key, proxy_req,
                               transport, proto, self._loop)
@@ -687,13 +689,19 @@ class ProxyConnector(TCPConnector):
                 raise
             else:
                 conn.detach()
-                if resp.status != 200:
-                    raise HttpProxyError(code=resp.status, message=resp.reason)
-                rawsock = transport.get_extra_info('socket', default=None)
-                if rawsock is None:
-                    raise RuntimeError(
-                        "Transport does not expose socket instance")
-                transport.pause_reading()
+                try:
+                    if resp.status != 200:
+                        raise HttpProxyError(code=resp.status,
+                                             message=resp.reason)
+                    rawsock = transport.get_extra_info('socket', default=None)
+                    if rawsock is None:
+                        raise RuntimeError(
+                            "Transport does not expose socket instance")
+                    # Duplicate the socket, so now we can close proxy transport
+                    rawsock = rawsock.dup()
+                finally:
+                    transport.close()
+
                 transport, proto = yield from self._loop.create_connection(
                     self._factory, ssl=self.ssl_context, sock=rawsock,
                     server_hostname=req.host)
@@ -703,12 +711,72 @@ class ProxyConnector(TCPConnector):
         return transport, proto
 
 
+class ProxyConnector(TCPConnector):
+    """Http Proxy connector.
+    Deprecated, use ClientSession.request with proxy parameters.
+    Is still here for backward compatibility.
+
+    proxy - Proxy URL address. Only HTTP proxy supported.
+    proxy_auth - (optional) Proxy HTTP Basic Auth
+    proxy_auth - aiohttp.helpers.BasicAuth
+    conn_timeout - (optional) Connect timeout.
+    keepalive_timeout - (optional) Keep-alive timeout.
+    force_close - Set to True to force close and do reconnect
+        after each request (and between redirects).
+    limit - The limit of simultaneous connections to the same endpoint.
+    loop - Optional event loop.
+
+    Usage:
+
+    >>> conn = ProxyConnector(proxy="http://some.proxy.com")
+    >>> session = ClientSession(connector=conn)
+    >>> resp = yield from session.get('http://python.org')
+
+    """
+
+    def __init__(self, proxy, *, proxy_auth=None, force_close=True,
+                 conn_timeout=None, keepalive_timeout=sentinel,
+                 limit=20, loop=None):
+        warnings.warn("ProxyConnector is deprecated, use "
+                      "client.get(url, proxy=proxy_url) instead",
+                      DeprecationWarning)
+        super().__init__(force_close=force_close,
+                         conn_timeout=conn_timeout,
+                         keepalive_timeout=keepalive_timeout,
+                         limit=limit, loop=loop)
+        proxy = URL(proxy)
+        self._proxy = proxy
+        self._proxy_auth = proxy_auth
+
+    @property
+    def proxy(self):
+        return self._proxy
+
+    @property
+    def proxy_auth(self):
+        return self._proxy_auth
+
+    @asyncio.coroutine
+    def _create_connection(self, req):
+        """
+        Use TCPConnector _create_connection, to emulate old ProxyConnector.
+        """
+        req.update_proxy(self._proxy, self._proxy_auth)
+        transport, proto = yield from super()._create_connection(req)
+
+        return transport, proto
+
+
 class UnixConnector(BaseConnector):
     """Unix socket connector.
 
-    :param str path: Unix socket path.
-    :param args: see :class:`BaseConnector`
-    :param kwargs: see :class:`BaseConnector`
+    path - Unix socket path.
+    conn_timeout - (optional) Connect timeout.
+    keepalive_timeout - (optional) Keep-alive timeout.
+    force_close - Set to True to force close and do reconnect
+        after each request (and between redirects).
+    limit - The limit of simultaneous connections to the same endpoint.
+    loop - Optional event loop.
 
     Usage:
 
@@ -718,8 +786,12 @@ class UnixConnector(BaseConnector):
 
     """
 
-    def __init__(self, path, **kwargs):
-        super().__init__(**kwargs)
+    def __init__(self, path, force_close=False, conn_timeout=None,
+                 keepalive_timeout=sentinel, limit=20, loop=None):
+        super().__init__(force_close=force_close,
+                         conn_timeout=conn_timeout,
+                         keepalive_timeout=keepalive_timeout,
+                         limit=limit, loop=loop)
         self._path = path
 
     @property
