@@ -1,8 +1,7 @@
-"""http related errors."""
+"""HTTP related errors."""
 
 from asyncio import TimeoutError
 
-
 __all__ = (
     'DisconnectedError', 'ClientDisconnectedError', 'ServerDisconnectedError',
 
@@ -15,7 +14,7 @@ __all__ = (
     'ClientRequestError', 'ClientResponseError',
     'FingerprintMismatch',
 
-    'WSServerHandshakeError', 'WSClientDisconnectedError')
+    'WSServerHandshakeError')
 
 
 class DisconnectedError(Exception):
@@ -30,16 +29,12 @@ class ServerDisconnectedError(DisconnectedError):
     """Server disconnected."""
 
 
-class WSClientDisconnectedError(ClientDisconnectedError):
-    """Deprecated."""
-
-
 class ClientError(Exception):
     """Base class for client connection errors."""
 
 
 class ClientHttpProcessingError(ClientError):
-    """Base class for client http processing errors."""
+    """Base class for client HTTP processing errors."""
 
 
 class ClientRequestError(ClientHttpProcessingError):
@@ -71,9 +66,9 @@ class ProxyConnectionError(ClientConnectionError):
 
 
 class HttpProcessingError(Exception):
-    """Http error.
+    """HTTP error.
 
-    Shortcut for raising http errors with custom code, message and headers.
+    Shortcut for raising HTTP errors with custom code, message and headers.
 
     :param int code: HTTP Error code.
     :param str message: (optional) Error message.
@@ -98,7 +93,7 @@ class WSServerHandshakeError(HttpProcessingError):
 
 
 class HttpProxyError(HttpProcessingError):
-    """Http proxy error.
+    """HTTP proxy error.
 
     Raised in :class:`aiohttp.connector.ProxyConnector` if
     proxy responds with status other than ``200 OK``
@@ -139,12 +134,14 @@ class LineTooLong(BadHttpMessage):
 
     def __init__(self, line, limit='Unknown'):
         super().__init__(
-            "got more than %s bytes when reading %s" % (limit, line))
+            "Got more than %s bytes when reading %s." % (limit, line))
 
 
 class InvalidHeader(BadHttpMessage):
 
     def __init__(self, hdr):
+        if isinstance(hdr, bytes):
+            hdr = hdr.decode('utf-8', 'surrogateescape')
         super().__init__('Invalid HTTP Header: {}'.format(hdr))
         self.hdr = hdr
 
@@ -158,11 +155,7 @@ class BadStatusLine(BadHttpMessage):
         self.line = line
 
 
-class ParserError(Exception):
-    """Base parser error."""
-
-
-class LineLimitExceededParserError(ParserError):
+class LineLimitExceededParserError(HttpBadRequest):
     """Line is too long."""
 
     def __init__(self, msg, limit):
@@ -183,7 +176,3 @@ class FingerprintMismatch(ClientConnectionError):
         return '<{} expected={} got={} host={} port={}>'.format(
             self.__class__.__name__, self.expected, self.got,
             self.host, self.port)
-
-
-class InvalidURL(Exception):
-    """Invalid URL."""
