@@ -2,16 +2,24 @@
 
 import asyncio
 import base64
+import binascii
+import cgi
 import datetime
 import functools
 import io
 import os
 import re
-from urllib.parse import quote, urlencode
-from collections import namedtuple
+import warnings
+from collections import MutableSequence, namedtuple
+from functools import total_ordering
+from pathlib import Path
+from time import gmtime, time
+from urllib.parse import urlencode
 
-from . import hdrs, multidict
-from .errors import InvalidURL
+from async_timeout import timeout
+from multidict import MultiDict, MultiDictProxy
+
+from . import hdrs
 
 try:
     from asyncio import ensure_future
@@ -19,7 +27,12 @@ except ImportError:
     ensure_future = asyncio.async
 
 
-__all__ = ('BasicAuth', 'FormData', 'parse_mimetype', 'Timeout')
+__all__ = ('BasicAuth', 'create_future', 'FormData', 'parse_mimetype',
+           'Timeout', 'ensure_future')
+
+
+sentinel = object()
+Timeout = timeout
 
 
 class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
@@ -37,14 +50,48 @@ class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
         if password is None:
             raise ValueError('None is not allowed as password value')
 
+        if ':' in login:
+            raise ValueError(
+                'A ":" is not allowed in login (RFC 1945#section-11.1)')
+
         return super().__new__(cls, login, password, encoding)
 
+    @classmethod
+    def decode(cls, auth_header, encoding='latin1'):
+        """Create a :class:`BasicAuth` object from an ``Authorization`` HTTP
+        header."""
+        split = auth_header.strip().split(' ')
+        if len(split) == 2:
+            if split[0].strip().lower() != 'basic':
+                raise ValueError('Unknown authorization method %s' % split[0])
+            to_decode = split[1]
+        else:
+            raise ValueError('Could not parse authorization header.')
+
+        try:
+            username, _, password = base64.b64decode(
+                to_decode.encode('ascii')
+            ).decode(encoding).partition(':')
+        except binascii.Error:
+            raise ValueError('Invalid base64 encoding.')
+
+        return cls(username, password, encoding=encoding)
+
     def encode(self):
         """Encode credentials."""
         creds = ('%s:%s' % (self.login, self.password)).encode(self.encoding)
         return 'Basic %s' % base64.b64encode(creds).decode(self.encoding)
 
 
+def create_future(loop):
+    """Compatibility wrapper for the loop.create_future() call introduced in
+    3.5.2."""
+    if hasattr(loop, 'create_future'):
+        return loop.create_future()
+    else:
+        return asyncio.Future(loop=loop)
+
+
 class FormData:
     """Helper class for multipart/form-data and
     application/x-www-form-urlencoded body generation."""
@@ -81,7 +128,7 @@ class FormData:
             if filename is None and content_transfer_encoding is None:
                 filename = name
 
-        type_options = multidict.MultiDict({'name': name})
+        type_options = MultiDict({'name': name})
         if filename is not None and not isinstance(filename, str):
             raise TypeError('filename must be an instance of str. '
                             'Got: %s' % filename)
@@ -117,9 +164,7 @@ class FormData:
                 k = guess_filename(rec, 'unknown')
                 self.add_field(k, rec)
 
-            elif isinstance(rec,
-                            (multidict.MultiDictProxy,
-                             multidict.MultiDict)):
+            elif isinstance(rec, (MultiDictProxy, MultiDict)):
                 to_add.extend(rec.items())
 
             elif isinstance(rec, (list, tuple)) and len(rec) == 2:
@@ -129,7 +174,8 @@ class FormData:
             else:
                 raise TypeError('Only io.IOBase, multidict and (name, file) '
                                 'pairs allowed, use .add_field() for passing '
-                                'more complex parameters')
+                                'more complex parameters, got {!r}'
+                                .format(rec))
 
     def _gen_form_urlencoded(self, encoding):
         # form data (x-www-form-urlencoded)
@@ -195,16 +241,10 @@ def parse_mimetype(mimetype):
     return mtype, stype, suffix, params
 
 
-def str_to_bytes(s, encoding='utf-8'):
-    if isinstance(s, str):
-        return s.encode(encoding)
-    return s
-
-
 def guess_filename(obj, default=None):
     name = getattr(obj, 'name', None)
     if name and name[0] != '<' and name[-1] != '>':
-        return os.path.split(name)[-1]
+        return Path(name).name
     return default
 
 
@@ -235,12 +275,29 @@ class AccessLogger:
         %{FOO}e  os.environ['FOO']
 
     """
+    LOG_FORMAT_MAP = {
+        'a': 'remote_address',
+        't': 'request_time',
+        'P': 'process_id',
+        'r': 'first_request_line',
+        's': 'response_status',
+        'b': 'response_size',
+        'O': 'bytes_sent',
+        'T': 'request_time',
+        'Tf': 'request_time_frac',
+        'D': 'request_time_micro',
+        'i': 'request_header',
+        'o': 'response_header',
+        'e': 'environ'
+    }
 
     LOG_FORMAT = '%a %l %u %t "%r" %s %b "%{Referrer}i" "%{User-Agent}i"'
-    FORMAT_RE = re.compile(r'%(\{([A-Za-z\-]+)\}([ioe])|[atPrsbOD]|Tf?)')
+    FORMAT_RE = re.compile(r'%(\{([A-Za-z0-9\-_]+)\}([ioe])|[atPrsbOD]|Tf?)')
     CLEANUP_RE = re.compile(r'(%[^s])')
     _FORMAT_CACHE = {}
 
+    KeyMethod = namedtuple('KeyMethod', 'key method')
+
     def __init__(self, logger, log_format=LOG_FORMAT):
         """Initialise the logger.
 
@@ -249,10 +306,12 @@ class AccessLogger:
 
         """
         self.logger = logger
+
         _compiled_format = AccessLogger._FORMAT_CACHE.get(log_format)
         if not _compiled_format:
             _compiled_format = self.compile_format(log_format)
             AccessLogger._FORMAT_CACHE[log_format] = _compiled_format
+
         self._log_format, self._methods = _compiled_format
 
     def compile_format(self, log_format):
@@ -279,33 +338,52 @@ class AccessLogger:
 
         log_format = log_format.replace("%l", "-")
         log_format = log_format.replace("%u", "-")
-        methods = []
+
+        # list of (key, method) tuples, we don't use an OrderedDict as users
+        # can repeat the same key more than once
+        methods = list()
 
         for atom in self.FORMAT_RE.findall(log_format):
             if atom[1] == '':
-                methods.append(getattr(AccessLogger, '_format_%s' % atom[0]))
+                format_key = self.LOG_FORMAT_MAP[atom[0]]
+                m = getattr(AccessLogger, '_format_%s' % atom[0])
             else:
+                format_key = (self.LOG_FORMAT_MAP[atom[2]], atom[1])
                 m = getattr(AccessLogger, '_format_%s' % atom[2])
-                methods.append(functools.partial(m, atom[1]))
+                m = functools.partial(m, atom[1])
+
+            methods.append(self.KeyMethod(format_key, m))
+
         log_format = self.FORMAT_RE.sub(r'%s', log_format)
         log_format = self.CLEANUP_RE.sub(r'%\1', log_format)
         return log_format, methods
 
     @staticmethod
     def _format_e(key, args):
-        return (args[1] or {}).get(multidict.upstr(key), '-')
+        return (args[1] or {}).get(key, '-')
 
     @staticmethod
     def _format_i(key, args):
-        return args[0].headers.get(multidict.upstr(key), '-')
+        if not args[0]:
+            return '(no headers)'
+
+        # suboptimal, make istr(key) once
+        return args[0].headers.get(key, '-')
 
     @staticmethod
     def _format_o(key, args):
-        return args[2].headers.get(multidict.upstr(key), '-')
+        # suboptimal, make istr(key) once
+        return args[2].headers.get(key, '-')
 
     @staticmethod
     def _format_a(args):
-        return args[3].get_extra_info('peername')[0]
+        if args[3] is None:
+            return '-'
+        peername = args[3].get_extra_info('peername')
+        if isinstance(peername, (list, tuple)):
+            return peername[0]
+        else:
+            return peername
 
     @staticmethod
     def _format_t(args):
@@ -348,7 +426,7 @@ class AccessLogger:
         return round(args[4] * 1000000)
 
     def _format_line(self, args):
-        return tuple(m(args) for m in self._methods)
+        return ((key, method(args)) for key, method in self._methods)
 
     def log(self, message, environ, response, transport, time):
         """Log access.
@@ -356,25 +434,34 @@ class AccessLogger:
         :param message: Request object. May be None.
         :param environ: Environment dict. May be None.
         :param response: Response object.
-        :param transport: Tansport object.
+        :param transport: Tansport object. May be None
         :param float time: Time taken to serve the request.
         """
         try:
-            self.logger.info(self._log_format % self._format_line(
-                [message, environ, response, transport, time]))
+            fmt_info = self._format_line(
+                [message, environ, response, transport, time])
+
+            values = list()
+            extra = dict()
+            for key, value in fmt_info:
+                values.append(value)
+
+                if key.__class__ is str:
+                    extra[key] = value
+                else:
+                    extra[key[0]] = {key[1]: value}
+
+            self.logger.info(self._log_format % tuple(values), extra=extra)
         except Exception:
             self.logger.exception("Error in logging")
 
 
-_marker = object()
-
-
 class reify:
     """Use as a class method decorator.  It operates almost exactly like
-    the Python ``@property`` decorator, but it puts the result of the
+    the Python `@property` decorator, but it puts the result of the
     method it decorates into the instance dict after the first call,
     effectively replacing the function it decorates with an instance
-    variable.  It is, in Python parlance, a non-data descriptor.
+    variable.  It is, in Python parlance, a data descriptor.
 
     """
 
@@ -383,109 +470,209 @@ class reify:
         try:
             self.__doc__ = wrapped.__doc__
         except:  # pragma: no cover
-            pass
+            self.__doc__ = ""
         self.name = wrapped.__name__
 
-    def __get__(self, inst, owner, _marker=_marker):
+    def __get__(self, inst, owner, _sentinel=sentinel):
         if inst is None:
             return self
-        val = inst.__dict__.get(self.name, _marker)
-        if val is not _marker:
+        val = inst._cache.get(self.name, _sentinel)
+        if val is not _sentinel:
             return val
         val = self.wrapped(inst)
-        inst.__dict__[self.name] = val
+        inst._cache[self.name] = val
         return val
 
     def __set__(self, inst, value):
         raise AttributeError("reified property is read-only")
 
 
-# The unreserved URI characters (RFC 3986)
-UNRESERVED_SET = frozenset(
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" +
-    "0123456789-._~")
+_ipv4_pattern = ('^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}'
+                 '(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$')
+_ipv6_pattern = (
+    '^(?:(?:(?:[A-F0-9]{1,4}:){6}|(?=(?:[A-F0-9]{0,4}:){0,6}'
+    '(?:[0-9]{1,3}\.){3}[0-9]{1,3}$)(([0-9A-F]{1,4}:){0,5}|:)'
+    '((:[0-9A-F]{1,4}){1,5}:|:)|::(?:[A-F0-9]{1,4}:){5})'
+    '(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.){3}'
+    '(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])|(?:[A-F0-9]{1,4}:){7}'
+    '[A-F0-9]{1,4}|(?=(?:[A-F0-9]{0,4}:){0,7}[A-F0-9]{0,4}$)'
+    '(([0-9A-F]{1,4}:){1,7}|:)((:[0-9A-F]{1,4}){1,7}|:)|(?:[A-F0-9]{1,4}:){7}'
+    ':|:(:[A-F0-9]{1,4}){7})$')
+_ipv4_regex = re.compile(_ipv4_pattern)
+_ipv6_regex = re.compile(_ipv6_pattern, flags=re.IGNORECASE)
+_ipv4_regexb = re.compile(_ipv4_pattern.encode('ascii'))
+_ipv6_regexb = re.compile(_ipv6_pattern.encode('ascii'), flags=re.IGNORECASE)
 
 
-def unquote_unreserved(uri):
-    """Un-escape any percent-escape sequences in a URI that are unreserved
-    characters. This leaves all reserved, illegal and non-ASCII bytes encoded.
-    """
-    parts = uri.split('%')
-    for i in range(1, len(parts)):
-        h = parts[i][0:2]
-        if len(h) == 2 and h.isalnum():
-            try:
-                c = chr(int(h, 16))
-            except ValueError:
-                raise InvalidURL("Invalid percent-escape sequence: '%s'" % h)
-
-            if c in UNRESERVED_SET:
-                parts[i] = c + parts[i][2:]
-            else:
-                parts[i] = '%' + parts[i]
+def is_ip_address(host):
+    if host is None:
+        return False
+    if isinstance(host, str):
+        if _ipv4_regex.match(host) or _ipv6_regex.match(host):
+            return True
         else:
-            parts[i] = '%' + parts[i]
-    return ''.join(parts)
+            return False
+    elif isinstance(host, (bytes, bytearray, memoryview)):
+        if _ipv4_regexb.match(host) or _ipv6_regexb.match(host):
+            return True
+        else:
+            return False
+    else:
+        raise TypeError("{} [{}] is not a str or bytes"
+                        .format(host, type(host)))
 
 
-def requote_uri(uri):
-    """Re-quote the given URI.
-
-    This function passes the given URI through an unquote/quote cycle to
-    ensure that it is fully and consistently quoted.
-    """
-    safe_with_percent = "!#$%&'()*+,/:;=?@[]~"
-    safe_without_percent = "!#$&'()*+,/:;=?@[]~"
-    try:
-        # Unquote only the unreserved characters
-        # Then quote only illegal characters (do not quote reserved,
-        # unreserved, or '%')
-        return quote(unquote_unreserved(uri), safe=safe_with_percent)
-    except InvalidURL:
-        # We couldn't unquote the given URI, so let's try quoting it, but
-        # there may be unquoted '%'s in the URI. We need to make sure they're
-        # properly quoted so they do not cause issues elsewhere.
-        return quote(uri, safe=safe_without_percent)
+def _get_kwarg(kwargs, old, new, value):
+    val = kwargs.pop(old, sentinel)
+    if val is not sentinel:
+        warnings.warn("{} is deprecated, use {} instead".format(old, new),
+                      DeprecationWarning,
+                      stacklevel=3)
+        return val
+    else:
+        return value
 
 
-class Timeout:
-    """Timeout context manager.
+@total_ordering
+class FrozenList(MutableSequence):
 
-    Useful in cases when you want to apply timeout logic around block
-    of code or in cases when asyncio.wait_for is not suitable. For example:
+    __slots__ = ('_frozen', '_items')
 
-    >>> with aiohttp.Timeout(0.001):
-    >>>     async with aiohttp.get('https://github.com') as r:
-    >>>         await r.text()
+    def __init__(self, items=None):
+        self._frozen = False
+        if items is not None:
+            items = list(items)
+        else:
+            items = []
+        self._items = items
+
+    def freeze(self):
+        self._frozen = True
+
+    def __getitem__(self, index):
+        return self._items[index]
+
+    def __setitem__(self, index, value):
+        if self._frozen:
+            raise RuntimeError("Cannot modify frozen list.")
+        self._items[index] = value
+
+    def __delitem__(self, index):
+        if self._frozen:
+            raise RuntimeError("Cannot modify frozen list.")
+        del self._items[index]
+
+    def __len__(self):
+        return self._items.__len__()
+
+    def __iter__(self):
+        return self._items.__iter__()
+
+    def __reversed__(self):
+        return self._items.__reversed__()
+
+    def __eq__(self, other):
+        return list(self) == other
+
+    def __le__(self, other):
+        return list(self) <= other
+
+    def insert(self, pos, item):
+        if self._frozen:
+            raise RuntimeError("Cannot modify frozen list.")
+        self._items.insert(pos, item)
 
 
-    :param timeout: timeout value in seconds
-    :param loop: asyncio compatible event loop
-    """
-    def __init__(self, timeout, *, loop=None):
-        self._timeout = timeout
-        if loop is None:
-            loop = asyncio.get_event_loop()
+class TimeService:
+    def __init__(self, loop):
         self._loop = loop
-        self._task = None
-        self._cancelled = False
-        self._cancel_handler = None
+        self._time = time()
+        self._strtime = None
+        self._count = 0
+        self._cb = loop.call_later(1, self._on_cb)
 
-    def __enter__(self):
-        self._task = asyncio.Task.current_task(loop=self._loop)
-        if self._task is None:
-            raise RuntimeError('Timeout context manager should be used '
-                               'inside a task')
-        self._cancel_handler = self._loop.call_later(
-            self._timeout, self._cancel_task)
-        return self
+    def stop(self):
+        if self._cb:
+            self._cb.cancel()
+        self._cb = None
+        self._loop = None
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        if exc_type is asyncio.CancelledError and self._cancelled:
-            self._task = None
-            raise asyncio.TimeoutError
-        self._cancel_handler.cancel()
-        self._task = None
+    def _on_cb(self):
+        self._count += 1
+        if self._count >= 10*60:
+            # reset timer every 10 minutes
+            self._count = 0
+            self._time = time()
+        else:
+            self._time += 1
+        self._strtime = None
+        self._cb = self._loop.call_later(1, self._on_cb)
 
-    def _cancel_task(self):
-        self._cancelled = self._task.cancel()
+    def _format_date_time(self):
+        # Weekday and month names for HTTP date/time formatting;
+        # always English!
+        # Tuples are contants stored in codeobject!
+        _weekdayname = ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
+        _monthname = (None,  # Dummy so we can use 1-based month numbers
+                      "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+                      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
+
+        year, month, day, hh, mm, ss, wd, y, z = gmtime(self._time)
+        return "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
+            _weekdayname[wd], day, _monthname[month], year, hh, mm, ss
+        )
+
+    def time(self):
+        return self._time
+
+    def strtime(self):
+        s = self._strtime
+        if s is None:
+            self._strtime = s = self._format_date_time()
+        return self._strtime
+
+
+class HeadersMixin:
+
+    _content_type = None
+    _content_dict = None
+    _stored_content_type = sentinel
+
+    def _parse_content_type(self, raw):
+        self._stored_content_type = raw
+        if raw is None:
+            # default value according to RFC 2616
+            self._content_type = 'application/octet-stream'
+            self._content_dict = {}
+        else:
+            self._content_type, self._content_dict = cgi.parse_header(raw)
+
+    @property
+    def content_type(self, *, _CONTENT_TYPE=hdrs.CONTENT_TYPE):
+        """The value of content part for Content-Type HTTP header."""
+        raw = self.headers.get(_CONTENT_TYPE)
+        if self._stored_content_type != raw:
+            self._parse_content_type(raw)
+        return self._content_type
+
+    @property
+    def charset(self, *, _CONTENT_TYPE=hdrs.CONTENT_TYPE):
+        """The value of charset part for Content-Type HTTP header."""
+        raw = self.headers.get(_CONTENT_TYPE)
+        if self._stored_content_type != raw:
+            self._parse_content_type(raw)
+        return self._content_dict.get('charset')
+
+    @property
+    def content_length(self, *, _CONTENT_LENGTH=hdrs.CONTENT_LENGTH):
+        """The value of Content-Length HTTP header."""
+        l = self.headers.get(_CONTENT_LENGTH)
+        if l is None:
+            return None
+        else:
+            return int(l)
+
+
+def check_loop(loop):
+    if loop is None:
+        loop = asyncio.get_event_loop()
