@@ -1,28 +1,25 @@
 import asyncio
-import binascii
 import base64
-import json
+import binascii
 import io
+import json
 import mimetypes
 import os
 import re
+import sys
 import uuid
 import warnings
 import zlib
-from urllib.parse import quote, unquote, urlencode, parse_qsl
-from collections import deque, Mapping, Sequence
+from collections import Mapping, Sequence, deque
+from pathlib import Path
+from urllib.parse import parse_qsl, quote, unquote, urlencode
 
+from multidict import CIMultiDict
+
+from .hdrs import (CONTENT_DISPOSITION, CONTENT_ENCODING, CONTENT_LENGTH,
+                   CONTENT_TRANSFER_ENCODING, CONTENT_TYPE)
 from .helpers import parse_mimetype
-from .multidict import CIMultiDict
 from .protocol import HttpParser
-from .hdrs import (
-    CONTENT_DISPOSITION,
-    CONTENT_ENCODING,
-    CONTENT_LENGTH,
-    CONTENT_TRANSFER_ENCODING,
-    CONTENT_TYPE
-)
-
 
 __all__ = ('MultipartReader', 'MultipartWriter',
            'BodyPartReader', 'BodyPartWriter',
@@ -36,6 +33,9 @@ SEPARATORS = {'(', ')', '<', '>', '@', ',', ';', ':', '\\', '"', '/', '[', ']',
               '?', '=', '{', '}', ' ', chr(9)}
 TOKEN = CHAR ^ CTL ^ SEPARATORS
 
+PY_35 = sys.version_info >= (3, 5)
+PY_352 = sys.version_info >= (3, 5, 2)
+
 
 class BadContentDispositionHeader(RuntimeWarning):
     pass
@@ -165,16 +165,19 @@ class MultipartResponseWrapper(object):
         self.resp = resp
         self.stream = stream
 
-    @asyncio.coroutine
-    def __aiter__(self):
-        return self
+    if PY_35:
+        def __aiter__(self):
+            return self
 
-    @asyncio.coroutine
-    def __anext__(self):
-        part = yield from self.next()
-        if part is None:
-            raise StopAsyncIteration  # NOQA
-        return part
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
+        @asyncio.coroutine
+        def __anext__(self):
+            part = yield from self.next()
+            if part is None:
+                raise StopAsyncIteration  # NOQA
+            return part
 
     def at_eof(self):
         """Returns ``True`` when all response data had been read.
@@ -212,17 +215,22 @@ class BodyPartReader(object):
         self._length = int(length) if length is not None else None
         self._read_bytes = 0
         self._unread = deque()
+        self._prev_chunk = None
+        self._content_eof = 0
 
-    @asyncio.coroutine
-    def __aiter__(self):
-        return self
+    if PY_35:
+        def __aiter__(self):
+            return self
 
-    @asyncio.coroutine
-    def __anext__(self):
-        part = yield from self.next()
-        if part is None:
-            raise StopAsyncIteration  # NOQA
-        return part
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
+        @asyncio.coroutine
+        def __anext__(self):
+            part = yield from self.next()
+            if part is None:
+                raise StopAsyncIteration  # NOQA
+            return part
 
     @asyncio.coroutine
     def next(self):
@@ -257,7 +265,6 @@ class BodyPartReader(object):
     @asyncio.coroutine
     def read_chunk(self, size=chunk_size):
         """Reads body part content chunk of the specified size.
-        The body part must has `Content-Length` header with proper value.
 
         :param int size: chunk size
 
@@ -265,17 +272,70 @@ class BodyPartReader(object):
         """
         if self._at_eof:
             return b''
+        if self._length:
+            chunk = yield from self._read_chunk_from_length(size)
+        else:
+            chunk = yield from self._read_chunk_from_stream(size)
+
+        self._read_bytes += len(chunk)
+        if self._read_bytes == self._length:
+            self._at_eof = True
+        if self._at_eof:
+            assert b'\r\n' == (yield from self._content.readline()), \
+                'reader did not read all the data or it is malformed'
+        return chunk
+
+    @asyncio.coroutine
+    def _read_chunk_from_length(self, size):
+        """Reads body part content chunk of the specified size.
+        The body part must has `Content-Length` header with proper value.
+
+        :param int size: chunk size
+
+        :rtype: bytearray
+        """
         assert self._length is not None, \
             'Content-Length required for chunked read'
         chunk_size = min(size, self._length - self._read_bytes)
         chunk = yield from self._content.read(chunk_size)
-        self._read_bytes += len(chunk)
-        if self._read_bytes == self._length:
-            self._at_eof = True
-            assert b'\r\n' == (yield from self._content.readline()), \
-                'reader did not read all the data or it is malformed'
         return chunk
 
+    @asyncio.coroutine
+    def _read_chunk_from_stream(self, size):
+        """Reads content chunk of body part with unknown length.
+        The `Content-Length` header for body part is not necessary.
+
+        :param int size: chunk size
+
+        :rtype: bytearray
+        """
+        assert size >= len(self._boundary) + 2, \
+            'Chunk size must be greater or equal than boundary length + 2'
+        first_chunk = self._prev_chunk is None
+        if first_chunk:
+            self._prev_chunk = yield from self._content.read(size)
+
+        chunk = yield from self._content.read(size)
+        self._content_eof += int(self._content.at_eof())
+        assert self._content_eof < 3, "Reading after EOF"
+        window = self._prev_chunk + chunk
+        sub = b'\r\n' + self._boundary
+        if first_chunk:
+            idx = window.find(sub)
+        else:
+            idx = window.find(sub, max(0, len(self._prev_chunk) - len(sub)))
+        if idx >= 0:
+            # pushing boundary back to content
+            self._content.unread_data(window[idx:])
+            if size > idx:
+                self._prev_chunk = self._prev_chunk[:idx]
+            chunk = window[len(self._prev_chunk):idx]
+            if not chunk:
+                self._at_eof = True
+        result = self._prev_chunk
+        self._prev_chunk = chunk
+        return result
+
     @asyncio.coroutine
     def readline(self):
         """Reads body part by line by line.
@@ -311,7 +371,7 @@ class BodyPartReader(object):
 
     @asyncio.coroutine
     def release(self):
-        """Lke :meth:`read`, but reads all the data to the void.
+        """Like :meth:`read`, but reads all the data to the void.
 
         :rtype: None
         """
@@ -326,7 +386,7 @@ class BodyPartReader(object):
 
     @asyncio.coroutine
     def text(self, *, encoding=None):
-        """Lke :meth:`read`, but assumes that body part contains text data.
+        """Like :meth:`read`, but assumes that body part contains text data.
 
         :param str encoding: Custom text encoding. Overrides specified
                              in charset param of `Content-Type` header
@@ -334,12 +394,14 @@ class BodyPartReader(object):
         :rtype: str
         """
         data = yield from self.read(decode=True)
-        encoding = encoding or self.get_charset(default='latin1')
+        # see https://www.w3.org/TR/html5/forms.html#multipart/form-data-encoding-algorithm # NOQA
+        # and https://dvcs.w3.org/hg/xhr/raw-file/tip/Overview.html#dom-xmlhttprequest-send # NOQA
+        encoding = encoding or self.get_charset(default='utf-8')
         return data.decode(encoding)
 
     @asyncio.coroutine
     def json(self, *, encoding=None):
-        """Lke :meth:`read`, but assumes that body parts contains JSON data.
+        """Like :meth:`read`, but assumes that body parts contains JSON data.
 
         :param str encoding: Custom JSON encoding. Overrides specified
                              in charset param of `Content-Type` header
@@ -352,7 +414,7 @@ class BodyPartReader(object):
 
     @asyncio.coroutine
     def form(self, *, encoding=None):
-        """Lke :meth:`read`, but assumes that body parts contains form
+        """Like :meth:`read`, but assumes that body parts contains form
         urlencoded data.
 
         :param str encoding: Custom form encoding. Overrides specified
@@ -379,7 +441,7 @@ class BodyPartReader(object):
         Supports ``gzip``, ``deflate`` and ``identity`` encodings for
         `Content-Encoding` header.
 
-        Supports ``base64``, ``quoted-printable`` encodings for
+        Supports ``base64``, ``quoted-printable``, ``binary`` encodings for
         `Content-Transfer-Encoding` header.
 
         :param bytearray data: Data to decode.
@@ -413,6 +475,8 @@ class BodyPartReader(object):
             return base64.b64decode(data)
         elif encoding == 'quoted-printable':
             return binascii.a2b_qp(data)
+        elif encoding == 'binary':
+            return data
         else:
             raise RuntimeError('unknown content transfer encoding: {}'
                                ''.format(encoding))
@@ -450,18 +514,22 @@ class MultipartReader(object):
         self._content = content
         self._last_part = None
         self._at_eof = False
+        self._at_bof = True
         self._unread = []
 
-    @asyncio.coroutine
-    def __aiter__(self):
-        return self
+    if PY_35:
+        def __aiter__(self):
+            return self
 
-    @asyncio.coroutine
-    def __anext__(self):
-        part = yield from self.next()
-        if part is None:
-            raise StopAsyncIteration  # NOQA
-        return part
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
+        @asyncio.coroutine
+        def __anext__(self):
+            part = yield from self.next()
+            if part is None:
+                raise StopAsyncIteration  # NOQA
+            return part
 
     @classmethod
     def from_response(cls, response):
@@ -484,10 +552,15 @@ class MultipartReader(object):
     @asyncio.coroutine
     def next(self):
         """Emits the next multipart body part."""
+        # So, if we're at BOF, we need to skip till the boundary.
         if self._at_eof:
             return
         yield from self._maybe_release_last_part()
-        yield from self._read_boundary()
+        if self._at_bof:
+            yield from self._read_until_first_boundary()
+            self._at_bof = False
+        else:
+            yield from self._read_boundary()
         if self._at_eof:  # we just read the last boundary, nothing to do there
             return
         self._last_part = yield from self.fetch_next_part()
@@ -545,6 +618,20 @@ class MultipartReader(object):
             return self._unread.pop()
         return (yield from self._content.readline())
 
+    @asyncio.coroutine
+    def _read_until_first_boundary(self):
+        while True:
+            chunk = yield from self._readline()
+            if chunk == b'':
+                raise ValueError("Could not find starting boundary %r"
+                                 % (self._boundary))
+            chunk = chunk.rstrip()
+            if chunk == self._boundary:
+                return
+            elif chunk == self._boundary + b'--':
+                self._at_eof = True
+                return
+
     @asyncio.coroutine
     def _read_boundary(self):
         chunk = (yield from self._readline()).rstrip()
@@ -558,10 +645,10 @@ class MultipartReader(object):
 
     @asyncio.coroutine
     def _read_headers(self):
-        lines = ['']
+        lines = [b'']
         while True:
             chunk = yield from self._content.readline()
-            chunk = chunk.decode().strip()
+            chunk = chunk.strip()
             lines.append(chunk)
             if not chunk:
                 break
@@ -652,7 +739,7 @@ class BodyPartWriter(object):
         if isinstance(obj, io.IOBase):
             name = getattr(obj, 'name', None)
             if name is not None:
-                return os.path.basename(name)
+                return Path(name).name
 
     def serialize(self):
         """Yields byte chunks for body part."""
@@ -767,6 +854,8 @@ class BodyPartWriter(object):
         elif encoding == 'quoted-printable':
             for chunk in stream:
                 yield binascii.b2a_qp(chunk)
+        elif encoding == 'binary':
+            yield from stream
         else:
             raise RuntimeError('unknown content transfer encoding: {}'
                                ''.format(encoding))
