@@ -7,14 +7,15 @@ import re
 import string
 import sys
 import zlib
-from abc import abstractmethod, ABCMeta
+from abc import ABC, abstractmethod
 from wsgiref.handlers import format_date_time
 
+from multidict import CIMultiDict, istr
+
 import aiohttp
+
 from . import errors, hdrs
-from .multidict import CIMultiDict, upstr
 from .log import internal_logger
-from .helpers import reify
 
 __all__ = ('HttpMessage', 'Request', 'Response',
            'HttpVersion', 'HttpVersion10', 'HttpVersion11',
@@ -25,8 +26,7 @@ __all__ = ('HttpMessage', 'Request', 'Response',
 ASCIISET = set(string.printable)
 METHRE = re.compile('[A-Z0-9$-_.]+')
 VERSRE = re.compile('HTTP/(\d+).(\d+)')
-HDRRE = re.compile('[\x00-\x1F\x7F()<>@,;:\[\]={} \t\\\\\"]')
-CONTINUATION = (' ', '\t')
+HDRRE = re.compile(b'[\x00-\x1F\x7F()<>@,;:\[\]={} \t\\\\\"]')
 EOF_MARKER = object()
 EOL_MARKER = object()
 STATUS_LINE_READY = object()
@@ -43,12 +43,14 @@ RawStatusLineMessage = collections.namedtuple(
 
 RawRequestMessage = collections.namedtuple(
     'RawRequestMessage',
-    ['method', 'path', 'version', 'headers', 'should_close', 'compression'])
+    ['method', 'path', 'version', 'headers', 'raw_headers',
+     'should_close', 'compression'])
 
 
 RawResponseMessage = collections.namedtuple(
     'RawResponseMessage',
-    ['version', 'code', 'reason', 'headers', 'should_close', 'compression'])
+    ['version', 'code', 'reason', 'headers', 'raw_headers',
+     'should_close', 'compression'])
 
 
 class HttpParser:
@@ -60,7 +62,7 @@ class HttpParser:
         self.max_field_size = max_field_size
 
     def parse_headers(self, lines):
-        """Parses RFC2822 headers from a stream.
+        """Parses RFC 5322 headers from a stream.
 
         Line continuations are supported. Returns list of header name
         and value pairs. Header name is in upper case.
@@ -68,6 +70,7 @@ class HttpParser:
         close_conn = None
         encoding = None
         headers = CIMultiDict()
+        raw_headers = []
 
         lines_idx = 1
         line = lines[1]
@@ -77,41 +80,48 @@ class HttpParser:
 
             # Parse initial header name : value pair.
             try:
-                name, value = line.split(':', 1)
+                bname, bvalue = line.split(b':', 1)
             except ValueError:
                 raise errors.InvalidHeader(line) from None
 
-            name = name.strip(' \t').upper()
-            if HDRRE.search(name):
-                raise errors.InvalidHeader(name)
+            bname = bname.strip(b' \t').upper()
+            if HDRRE.search(bname):
+                raise errors.InvalidHeader(bname)
 
             # next line
             lines_idx += 1
             line = lines[lines_idx]
 
             # consume continuation lines
-            continuation = line and line[0] in CONTINUATION
+            continuation = line and line[0] in (32, 9)  # (' ', '\t')
 
             if continuation:
-                value = [value]
+                bvalue = [bvalue]
                 while continuation:
                     header_length += len(line)
                     if header_length > self.max_field_size:
                         raise errors.LineTooLong(
-                            'limit request headers fields size')
-                    value.append(line)
+                            'request header field {}'.format(
+                                bname.decode("utf8", "xmlcharrefreplace")),
+                            self.max_field_size)
+                    bvalue.append(line)
 
                     # next line
                     lines_idx += 1
                     line = lines[lines_idx]
-                    continuation = line[0] in CONTINUATION
-                value = '\r\n'.join(value)
+                    continuation = line[0] in (32, 9)  # (' ', '\t')
+                bvalue = b'\r\n'.join(bvalue)
             else:
                 if header_length > self.max_field_size:
                     raise errors.LineTooLong(
-                        'limit request headers fields size')
+                        'request header field {}'.format(
+                            bname.decode("utf8", "xmlcharrefreplace")),
+                        self.max_field_size)
 
-            value = value.strip()
+            bvalue = bvalue.strip()
+
+            name = istr(bname.decode('utf-8', 'surrogateescape'))
+            value = bvalue.decode('utf-8', 'surrogateescape')
 
             # keep-alive and encoding
             if name == hdrs.CONNECTION:
@@ -126,8 +136,9 @@ class HttpParser:
                     encoding = enc
 
             headers.add(name, value)
+            raw_headers.append((bname, bvalue))
 
-        return headers, close_conn, encoding
+        return headers, raw_headers, close_conn, encoding
 
 
 class HttpPrefixParser:
@@ -160,18 +171,17 @@ class HttpRequestParser(HttpParser):
     """
 
     def __call__(self, out, buf):
-        # read http message (request line + headers)
+        # read HTTP message (request line + headers)
         try:
             raw_data = yield from buf.readuntil(
                 b'\r\n\r\n', self.max_headers)
         except errors.LineLimitExceededParserError as exc:
-            raise errors.LineTooLong(exc.limit) from None
+            raise errors.LineTooLong('request header', exc.limit) from None
 
-        lines = raw_data.decode(
-            'utf-8', 'surrogateescape').split('\r\n')
+        lines = raw_data.split(b'\r\n')
 
         # request line
-        line = lines[0]
+        line = lines[0].decode('utf-8', 'surrogateescape')
         try:
             method, path, version = line.split(None, 2)
         except ValueError:
@@ -193,7 +203,7 @@ class HttpRequestParser(HttpParser):
             raise errors.BadStatusLine(version)
 
         # read headers
-        headers, close, compression = self.parse_headers(lines)
+        headers, raw_headers, close, compression = self.parse_headers(lines)
         if close is None:  # then the headers weren't set in the request
             if version <= HttpVersion10:  # HTTP 1.0 must asks to not close
                 close = True
@@ -202,7 +212,8 @@ class HttpRequestParser(HttpParser):
 
         out.feed_data(
             RawRequestMessage(
-                method, path, version, headers, close, compression),
+                method, path, version, headers, raw_headers,
+                close, compression),
             len(raw_data))
         out.feed_eof()
 
@@ -214,17 +225,16 @@ class HttpResponseParser(HttpParser):
     Returns RawResponseMessage"""
 
     def __call__(self, out, buf):
-        # read http message (response line + headers)
+        # read HTTP message (response line + headers)
         try:
             raw_data = yield from buf.readuntil(
                 b'\r\n\r\n', self.max_line_size + self.max_headers)
         except errors.LineLimitExceededParserError as exc:
-            raise errors.LineTooLong(exc.limit) from None
+            raise errors.LineTooLong('response header', exc.limit) from None
 
-        lines = raw_data.decode(
-            'utf-8', 'surrogateescape').split('\r\n')
+        lines = raw_data.split(b'\r\n')
 
-        line = lines[0]
+        line = lines[0].decode('utf-8', 'surrogateescape')
         try:
             version, status = line.split(None, 1)
         except ValueError:
@@ -251,7 +261,7 @@ class HttpResponseParser(HttpParser):
             raise errors.BadStatusLine(line)
 
         # read headers
-        headers, close, compression = self.parse_headers(lines)
+        headers, raw_headers, close, compression = self.parse_headers(lines)
 
         if close is None:
             close = version <= HttpVersion10
@@ -259,7 +269,7 @@ class HttpResponseParser(HttpParser):
         out.feed_data(
             RawResponseMessage(
                 version, status, reason.strip(),
-                headers, close, compression),
+                headers, raw_headers, close, compression),
             len(raw_data))
         out.feed_eof()
 
@@ -281,7 +291,8 @@ class HttpPayloadParser:
             length = 8
 
         # payload decompression wrapper
-        if self.compression and self.message.compression:
+        if (self.response_with_body and
+                self.compression and self.message.compression):
             out = DeflateBuffer(out, self.message.compression)
 
         # payload parser
@@ -476,7 +487,7 @@ def filter_pipe(filter, filter2, *,
         chunk = yield EOL_MARKER
 
 
-class HttpMessage(metaclass=ABCMeta):
+class HttpMessage(ABC):
     """HttpMessage allows to write headers and payload to a stream.
 
     For example, lets say we want to read file then compress it with deflate
@@ -547,6 +558,7 @@ class HttpMessage(metaclass=ABCMeta):
         self.output_length = 0
         self.headers_length = 0
         self._output_size = 0
+        self._cache = {}
 
     @property
     @abstractmethod
@@ -602,7 +614,7 @@ class HttpMessage(metaclass=ABCMeta):
             'Header {!r} should have string value, got {!r}'.format(
                 name, value)
 
-        name = upstr(name)
+        name = istr(name)
         value = value.strip()
 
         if name == hdrs.CONTENT_LENGTH:
@@ -632,7 +644,7 @@ class HttpMessage(metaclass=ABCMeta):
             self.headers.add(name, value)
 
     def add_headers(self, *headers):
-        """Adds headers to a http message."""
+        """Adds headers to a HTTP message."""
         for name, value in headers:
             self.add_header(name, value)
 
@@ -670,14 +682,18 @@ class HttpMessage(metaclass=ABCMeta):
 
     def _add_default_headers(self):
         # set the connection header
+        connection = None
         if self.upgrade:
             connection = 'upgrade'
         elif not self.closing if self.keepalive is None else self.keepalive:
-            connection = 'keep-alive'
+            if self.version == HttpVersion10:
+                connection = 'keep-alive'
         else:
-            connection = 'close'
+            if self.version == HttpVersion11:
+                connection = 'close'
 
-        self.headers[hdrs.CONNECTION] = connection
+        if connection is not None:
+            self.headers[hdrs.CONNECTION] = connection
 
     def write(self, chunk, *,
               drain=False, EOF_MARKER=EOF_MARKER, EOL_MARKER=EOL_MARKER):
@@ -696,8 +712,6 @@ class HttpMessage(metaclass=ABCMeta):
         if self._send_headers and not self.headers_sent:
             self.send_headers()
 
-        assert self.writer is not None, 'send_headers() is not called.'
-
         if self.filter:
             chunk = self.filter.send(chunk)
             while chunk not in (EOF_MARKER, EOL_MARKER):
@@ -814,11 +828,11 @@ class HttpMessage(metaclass=ABCMeta):
 
 
 class Response(HttpMessage):
-    """Create http response message.
+    """Create HTTP response message.
 
     Transport is a socket stream transport. status is a response status code,
     status has to be integer value. http_version is a tuple that represents
-    http version, (1, 0) stands for HTTP/1.0 and (1, 1) is for HTTP/1.1
+    HTTP version, (1, 0) stands for HTTP/1.0 and (1, 1) is for HTTP/1.1
     """
 
     HOP_HEADERS = ()
@@ -850,7 +864,7 @@ class Response(HttpMessage):
     def reason(self):
         return self._reason
 
-    @reify
+    @property
     def status_line(self):
         version = self.version
         return 'HTTP/{}.{} {} {}\r\n'.format(
@@ -858,7 +872,7 @@ class Response(HttpMessage):
 
     def autochunked(self):
         return (self.length is None and
-                self.version >= HttpVersion11)
+                self._version >= HttpVersion11)
 
     def _add_default_headers(self):
         super()._add_default_headers()
@@ -869,15 +883,21 @@ class Response(HttpMessage):
         self.headers.setdefault(hdrs.SERVER, self.SERVER_SOFTWARE)
 
 
+class WebResponse(Response):
+    """For usage in aiohttp.web only"""
+    def _add_default_headers(self):
+        pass
+
+
 class Request(HttpMessage):
 
     HOP_HEADERS = ()
 
     def __init__(self, transport, method, path,
                  http_version=HttpVersion11, close=False):
-        # set the default for HTTP 1.0 to be different
+        # set the default for HTTP 0.9 to be different
         # will only be overwritten with keep-alive header
-        if http_version < HttpVersion11:
+        if http_version < HttpVersion10:
             close = True
 
         super().__init__(transport, http_version, close)
@@ -893,12 +913,12 @@ class Request(HttpMessage):
     def path(self):
         return self._path
 
-    @reify
+    @property
     def status_line(self):
         return '{0} {1} HTTP/{2[0]}.{2[1]}\r\n'.format(
             self.method, self.path, self.version)
 
     def autochunked(self):
         return (self.length is None and
-                self.version >= HttpVersion11 and
+                self._version >= HttpVersion11 and
                 self.status not in (304, 204))
