@@ -1,17 +1,17 @@
-"""simple http server."""
+"""simple HTTP server."""
 
 import asyncio
 import http.server
-import traceback
 import socket
-
+import traceback
+import warnings
+from contextlib import suppress
 from html import escape as html_escape
-from math import ceil
 
 import aiohttp
-from aiohttp import errors, streams, hdrs, helpers
-from aiohttp.log import server_logger
-from aiohttp.helpers import ensure_future
+from aiohttp import errors, hdrs, helpers, streams
+from aiohttp.helpers import Timeout, _get_kwarg, ensure_future
+from aiohttp.log import access_logger, server_logger
 
 __all__ = ('ServerHttpProtocol',)
 
@@ -41,9 +41,9 @@ EMPTY_PAYLOAD = streams.EmptyStreamReader()
 
 
 class ServerHttpProtocol(aiohttp.StreamProtocol):
-    """Simple http protocol implementation.
+    """Simple HTTP protocol implementation.
 
-    ServerHttpProtocol handles incoming http request. It reads request line,
+    ServerHttpProtocol handles incoming HTTP request. It reads request line,
     request headers and request payload and calls handle_request() method.
     By default it always returns with 404 response.
 
@@ -51,16 +51,13 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
     status line, bad headers or incomplete payload. If any error occurs,
     connection gets closed.
 
-    :param keep_alive: number of seconds before closing keep-alive connection
-    :type keep_alive: int or None
+    :param keepalive_timeout: number of seconds before closing
+                              keep-alive connection
+    :type keepalive_timeout: int or None
 
-    :param bool keep_alive_on: keep-alive is o, default is on
+    :param bool tcp_keepalive: TCP keep-alive is on, default is on
 
-    :param int timeout: slow request timeout
-
-    :param allowed_methods: (optional) List of allowed request methods.
-                            Set to empty list to allow all methods.
-    :type allowed_methods: tuple
+    :param int slow_request_timeout: slow request timeout
 
     :param bool debug: enable debug mode
 
@@ -73,37 +70,64 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
     :param str access_log_format: access log format string
 
     :param loop: Optional event loop
+
+    :param int max_line_size: Optional maximum header line size
+
+    :param int max_field_size: Optional maximum header field size
+
+    :param int max_headers: Optional maximum header size
+
     """
     _request_count = 0
     _request_handler = None
     _reading_request = False
-    _keep_alive = False  # keep transport open
-    _keep_alive_handle = None  # keep alive timer handle
-    _timeout_handle = None  # slow request timer handle
-
-    _request_prefix = aiohttp.HttpPrefixParser()  # http method parser
-    _request_parser = aiohttp.HttpRequestParser()  # default request parser
+    _keepalive = False  # keep transport open
 
     def __init__(self, *, loop=None,
-                 keep_alive=75,  # NGINX default value is 75 secs
-                 keep_alive_on=True,
-                 timeout=0,
+                 keepalive_timeout=75,  # NGINX default value is 75 secs
+                 tcp_keepalive=True,
+                 slow_request_timeout=0,
                  logger=server_logger,
-                 access_log=None,
+                 access_log=access_logger,
                  access_log_format=helpers.AccessLogger.LOG_FORMAT,
                  debug=False,
-                 log=None,
+                 max_line_size=8190,
+                 max_headers=32768,
+                 max_field_size=8190,
+                 lingering_time=30,
+                 lingering_timeout=5,
                  **kwargs):
+
+        # process deprecated params
+        logger = _get_kwarg(kwargs, 'log', 'logger', logger)
+
+        tcp_keepalive = _get_kwarg(kwargs, 'keep_alive_on',
+                                   'tcp_keepalive', tcp_keepalive)
+
+        keepalive_timeout = _get_kwarg(kwargs, 'keep_alive',
+                                       'keepalive_timeout', keepalive_timeout)
+
+        slow_request_timeout = _get_kwarg(kwargs, 'timeout',
+                                          'slow_request_timeout',
+                                          slow_request_timeout)
+
         super().__init__(
             loop=loop,
             disconnect_error=errors.ClientDisconnectedError, **kwargs)
 
-        self._keep_alive_on = keep_alive_on
-        self._keep_alive_period = keep_alive  # number of seconds to keep alive
-        self._timeout = timeout  # slow request timeout
+        self._tcp_keepalive = tcp_keepalive
+        self._keepalive_timeout = keepalive_timeout
+        self._slow_request_timeout = slow_request_timeout
+        self._lingering_time = float(lingering_time)
+        self._lingering_timeout = float(lingering_timeout)
         self._loop = loop if loop is not None else asyncio.get_event_loop()
 
-        self.logger = log or logger
+        self._request_parser = aiohttp.HttpRequestParser(
+            max_line_size=max_line_size,
+            max_field_size=max_field_size,
+            max_headers=max_headers)
+
+        self.logger = logger
         self.debug = debug
         self.access_log = access_log
         if access_log:
@@ -111,81 +135,68 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
                                                       access_log_format)
         else:
             self.access_logger = None
+        self._closing = False
 
     @property
     def keep_alive_timeout(self):
-        return self._keep_alive_period
+        warnings.warn("Use keepalive_timeout property instead",
+                      DeprecationWarning,
+                      stacklevel=2)
+        return self._keepalive_timeout
 
-    def closing(self, timeout=15.0):
+    @property
+    def keepalive_timeout(self):
+        return self._keepalive_timeout
+
+    @asyncio.coroutine
+    def shutdown(self, timeout=15.0):
         """Worker process is about to exit, we need cleanup everything and
         stop accepting requests. It is especially important for keep-alive
         connections."""
-        self._keep_alive = False
-        self._keep_alive_on = False
-        self._keep_alive_period = None
+        if self._request_handler is None:
+            return
+        self._closing = True
 
-        if (not self._reading_request and self.transport is not None):
-            if self._request_handler:
-                self._request_handler.cancel()
-                self._request_handler = None
-
-            self.transport.close()
-            self.transport = None
-        elif self.transport is not None and timeout:
-            if self._timeout_handle is not None:
-                self._timeout_handle.cancel()
-
-            # use slow request timeout for closing
-            # connection_lost cleans timeout handler
-            now = self._loop.time()
-            self._timeout_handle = self._loop.call_at(
-                ceil(now+timeout), self.cancel_slow_request)
+        if self._request_count > 1 and not self._reading_request:
+            # force-close idle keep-alive connections
+            self._request_handler.cancel()
+        elif timeout:
+            canceller = self._loop.call_later(timeout,
+                                              self._request_handler.cancel)
+            with suppress(asyncio.CancelledError):
+                yield from self._request_handler
+            canceller.cancel()
+        else:
+            self._request_handler.cancel()
 
     def connection_made(self, transport):
         super().connection_made(transport)
 
         self._request_handler = ensure_future(self.start(), loop=self._loop)
 
-        # start slow request timer
-        if self._timeout:
-            now = self._loop.time()
-            self._timeout_handle = self._loop.call_at(
-                ceil(now+self._timeout), self.cancel_slow_request)
-
-        if self._keep_alive_on:
+        if self._tcp_keepalive:
             tcp_keepalive(self, transport)
 
     def connection_lost(self, exc):
         super().connection_lost(exc)
 
+        self._closing = True
         if self._request_handler is not None:
             self._request_handler.cancel()
-            self._request_handler = None
-        if self._keep_alive_handle is not None:
-            self._keep_alive_handle.cancel()
-            self._keep_alive_handle = None
-        if self._timeout_handle is not None:
-            self._timeout_handle.cancel()
-            self._timeout_handle = None
 
     def data_received(self, data):
         super().data_received(data)
 
-        # reading request
-        if not self._reading_request:
-            self._reading_request = True
-
-        # stop keep-alive timer
-        if self._keep_alive_handle is not None:
-            self._keep_alive_handle.cancel()
-            self._keep_alive_handle = None
+        # we can not gracefully shutdown handler
+        # if we in process of reading request
+        self._reading_request = True
 
     def keep_alive(self, val):
         """Set keep-alive connection mode.
 
         :param bool val: new state.
         """
-        self._keep_alive = val
+        self._keepalive = val
 
     def log_access(self, message, environ, response, time):
         if self.access_logger:
@@ -199,16 +210,6 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
     def log_exception(self, *args, **kw):
         self.logger.exception(*args, **kw)
 
-    def cancel_slow_request(self):
-        if self._request_handler is not None:
-            self._request_handler.cancel()
-            self._request_handler = None
-
-        if self.transport is not None:
-            self.transport.close()
-
-        self.log_debug('Close slow request.')
-
     @asyncio.coroutine
     def start(self):
         """Start processing of incoming requests.
@@ -220,39 +221,33 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
         keep_alive(True) specified.
         """
         reader = self.reader
+        self.writer.set_tcp_nodelay(True)
 
-        while True:
-            message = None
-            self._keep_alive = False
-            self._request_count += 1
-            self._reading_request = False
+        try:
+            while not self._closing:
+                message = None
+                self._keepalive = False
+                self._request_count += 1
+                self._reading_request = False
 
-            payload = None
-            try:
-                # read http request method
-                prefix = reader.set_parser(self._request_prefix)
-                yield from prefix.read()
+                payload = None
 
-                # start reading request
-                self._reading_request = True
-
-                # start slow request timer
-                if self._timeout and self._timeout_handle is None:
-                    now = self._loop.time()
-                    self._timeout_handle = self._loop.call_at(
-                        ceil(now+self._timeout), self.cancel_slow_request)
-
-                # read request headers
-                httpstream = reader.set_parser(self._request_parser)
-                message = yield from httpstream.read()
-
-                # cancel slow request timer
-                if self._timeout_handle is not None:
-                    self._timeout_handle.cancel()
-                    self._timeout_handle = None
+                # slow request timer
+                with Timeout(max(self._slow_request_timeout,
+                                 self._keepalive_timeout), loop=self._loop):
+                    # read request headers
+                    httpstream = reader.set_parser(self._request_parser)
+                    message = yield from httpstream.read()
 
                 # request may not have payload
-                if (message.headers.get(hdrs.CONTENT_LENGTH, 0) or
+                try:
+                    content_length = int(
+                        message.headers.get(hdrs.CONTENT_LENGTH, 0))
+                except ValueError:
+                    raise errors.InvalidHeader(hdrs.CONTENT_LENGTH) from None
+
+                if (content_length > 0 or
+                    message.method == hdrs.METH_CONNECT or
                     hdrs.SEC_WEBSOCKET_KEY1 in message.headers or
                     'chunked' in message.headers.get(
                         hdrs.TRANSFER_ENCODING, '')):
@@ -265,65 +260,65 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
 
                 yield from self.handle_request(message, payload)
 
-            except asyncio.CancelledError:
-                return
-            except errors.ClientDisconnectedError:
-                self.log_debug(
-                    'Ignored premature client disconnection #1.')
-                return
-            except errors.HttpProcessingError as exc:
-                if self.transport is not None:
-                    yield from self.handle_error(exc.code, message,
-                                                 None, exc, exc.headers,
-                                                 exc.message)
-            except errors.LineLimitExceededParserError as exc:
-                yield from self.handle_error(400, message, None, exc)
-            except Exception as exc:
-                yield from self.handle_error(500, message, None, exc)
-            finally:
-                if self.transport is None:
-                    self.log_debug(
-                        'Ignored premature client disconnection #2.')
-                    return
-
-                if payload and not payload.is_eof():
+                if not payload.is_eof():
                     self.log_debug('Uncompleted request.')
-                    self._request_handler = None
-                    self.transport.close()
-                    return
+                    self._closing = True
+
+                    if self._lingering_time:
+                        self.transport.write_eof()
+                        self.log_debug(
+                            'Start lingering close timer for %s sec.',
+                            self._lingering_time)
+
+                        end_time = self._loop.time() + self._lingering_time
+
+                        with suppress(asyncio.TimeoutError,
+                                      errors.ClientDisconnectedError):
+                            while self._loop.time() < end_time:
+                                with Timeout(self._lingering_timeout,
+                                             loop=self._loop):
+                                    # read and ignore
+                                    yield from payload.readany()
                 else:
                     reader.unset_parser()
+                    if not self._keepalive or not self._keepalive_timeout:
+                        self._closing = True
 
-                if self._request_handler:
-                    if self._keep_alive and self._keep_alive_period:
-                        self.log_debug(
-                            'Start keep-alive timer for %s sec.',
-                            self._keep_alive_period)
-                        now = self._loop.time()
-                        self._keep_alive_handle = self._loop.call_at(
-                            ceil(now+self._keep_alive_period),
-                            self.transport.close)
-                    elif self._keep_alive and self._keep_alive_on:
-                        # do nothing, rely on kernel or upstream server
-                        pass
-                    else:
-                        self.log_debug('Close client connection.')
-                        self._request_handler = None
-                        self.transport.close()
-                        return
-                else:
-                    # connection is closed
-                    return
+        except asyncio.CancelledError:
+            self.log_debug(
+                'Request handler cancelled.')
+            return
+        except asyncio.TimeoutError:
+            self.log_debug(
+                'Request handler timed out.')
+            return
+        except errors.ClientDisconnectedError:
+            self.log_debug(
+                'Ignored premature client disconnection #1.')
+            return
+        except errors.HttpProcessingError as exc:
+            yield from self.handle_error(exc.code, message,
+                                         None, exc, exc.headers,
+                                         exc.message)
+        except Exception as exc:
+            yield from self.handle_error(500, message, None, exc)
+        finally:
+            self._request_handler = None
+            if self.transport is None:
+                self.log_debug(
+                    'Ignored premature client disconnection #2.')
+            else:
+                self.transport.close()
 
     def handle_error(self, status=500, message=None,
                      payload=None, exc=None, headers=None, reason=None):
         """Handle errors.
 
-        Returns http response with specific status code. Logs additional
+        Returns HTTP response with specific status code. Logs additional
         information. It always closes current connection."""
         now = self._loop.time()
         try:
-            if self._request_handler is None:
+            if self.transport is None:
                 # client has been disconnected during writing.
                 return ()
 
@@ -369,7 +364,7 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
             self.keep_alive(False)
 
     def handle_request(self, message, payload):
-        """Handle a single http request.
+        """Handle a single HTTP request.
 
         Subclass should override this method. By default it always
         returns 404 response.
