@@ -1,9 +1,10 @@
-import sys
 import asyncio
 import collections
 import functools
+import sys
 import traceback
 
+from . import helpers
 from .log import internal_logger
 
 __all__ = (
@@ -12,8 +13,8 @@ __all__ = (
     'FlowControlDataQueue', 'FlowControlChunksQueue')
 
 PY_35 = sys.version_info >= (3, 5)
+PY_352 = sys.version_info >= (3, 5, 2)
 
-EOF_MARKER = b''
 DEFAULT_LIMIT = 2 ** 16
 
 
@@ -21,50 +22,56 @@ class EofStream(Exception):
     """eof stream indication."""
 
 
-class AsyncStreamIterator:
+if PY_35:
+    class AsyncStreamIterator:
 
-    def __init__(self, read_func):
-        self.read_func = read_func
+        def __init__(self, read_func):
+            self.read_func = read_func
 
-    @asyncio.coroutine
-    def __aiter__(self):
-        return self
+        def __aiter__(self):
+            return self
 
-    @asyncio.coroutine
-    def __anext__(self):
-        try:
-            rv = yield from self.read_func()
-        except EofStream:
-            raise StopAsyncIteration  # NOQA
-        if rv == EOF_MARKER:
-            raise StopAsyncIteration  # NOQA
-        return rv
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
+        @asyncio.coroutine
+        def __anext__(self):
+            try:
+                rv = yield from self.read_func()
+            except EofStream:
+                raise StopAsyncIteration  # NOQA
+            if rv == b'':
+                raise StopAsyncIteration  # NOQA
+            return rv
 
 
 class AsyncStreamReaderMixin:
 
     if PY_35:
-        @asyncio.coroutine
         def __aiter__(self):
             return AsyncStreamIterator(self.readline)
 
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
         def iter_chunked(self, n):
             """Returns an asynchronous iterator that yields chunks of size n.
 
-            .. versionadded:: Python-3.5 available for Python 3.5+ only
+            Python-3.5 available for Python 3.5+ only
             """
             return AsyncStreamIterator(lambda: self.read(n))
 
         def iter_any(self):
-            """Returns an asynchronous iterator that yields slices of data as they come.
+            """Returns an asynchronous iterator that yields slices of data
+            as they come.
 
-            .. versionadded:: Python-3.5 available for Python 3.5+ only
+            Python-3.5 available for Python 3.5+ only
             """
             return AsyncStreamIterator(self.readany)
 
 
-class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
-    """An enhancement of :class:`asyncio.StreamReader`.
+class StreamReader(AsyncStreamReaderMixin):
+    """An enhancement of asyncio.StreamReader.
 
     Supports asynchronous iteration by line, chunk or as available::
 
@@ -75,13 +82,11 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
         async for slice in reader.iter_any():
             ...
 
-    .. automethod:: AsyncStreamReaderMixin.iter_chunked
-    .. automethod:: AsyncStreamReaderMixin.iter_any
     """
 
     total_bytes = 0
 
-    def __init__(self, limit=DEFAULT_LIMIT, loop=None):
+    def __init__(self, limit=DEFAULT_LIMIT, timeout=None, loop=None):
         self._limit = limit
         if loop is None:
             loop = asyncio.get_event_loop()
@@ -91,8 +96,10 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
         self._buffer_offset = 0
         self._eof = False
         self._waiter = None
+        self._canceller = None
         self._eof_waiter = None
         self._exception = None
+        self._timeout = timeout
 
     def __repr__(self):
         info = ['StreamReader']
@@ -120,6 +127,11 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_exception(exc)
 
+        canceller = self._canceller
+        if canceller is not None:
+            self._canceller = None
+            canceller.cancel()
+
     def feed_eof(self):
         self._eof = True
 
@@ -129,6 +141,11 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_result(True)
 
+        canceller = self._canceller
+        if canceller is not None:
+            self._canceller = None
+            canceller.cancel()
+
         waiter = self._eof_waiter
         if waiter is not None:
             self._eof_waiter = None
@@ -149,12 +166,24 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             return
 
         assert self._eof_waiter is None
-        self._eof_waiter = asyncio.Future(loop=self._loop)
+        self._eof_waiter = helpers.create_future(self._loop)
         try:
             yield from self._eof_waiter
         finally:
             self._eof_waiter = None
 
+    def unread_data(self, data):
+        """ rollback reading some data from stream, inserting it to buffer head.
+        """
+        if not data:
+            return
+
+        if self._buffer_offset:
+            self._buffer[0] = self._buffer[0][self._buffer_offset:]
+            self._buffer_offset = 0
+        self._buffer.appendleft(data)
+        self._buffer_size += len(data)
+
     def feed_data(self, data):
         assert not self._eof, 'feed_data after feed_eof'
 
@@ -171,7 +200,13 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_result(False)
 
-    def _create_waiter(self, func_name):
+        canceller = self._canceller
+        if canceller is not None:
+            self._canceller = None
+            canceller.cancel()
+
+    @asyncio.coroutine
+    def _wait(self, func_name):
         # StreamReader uses a future to link the protocol feed_data() method
         # to a read coroutine. Running two read coroutines at the same time
         # would have an unexpected behaviour. It would not possible to know
@@ -179,7 +214,18 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
         if self._waiter is not None:
             raise RuntimeError('%s() called while another coroutine is '
                                'already waiting for incoming data' % func_name)
-        return asyncio.Future(loop=self._loop)
+        waiter = self._waiter = helpers.create_future(self._loop)
+        if self._timeout:
+            self._canceller = self._loop.call_later(self._timeout,
+                                                    self.set_exception,
+                                                    asyncio.TimeoutError())
+        try:
+            yield from waiter
+        finally:
+            self._waiter = None
+            if self._canceller is not None:
+                self._canceller.cancel()
+                self._canceller = None
 
     @asyncio.coroutine
     def readline(self):
@@ -195,7 +241,7 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
                 offset = self._buffer_offset
                 ichar = self._buffer[0].find(b'\n', offset) + 1
                 # Read from current offset to found b'\n' or to the end.
-                data = self._read_nowait(ichar - offset if ichar else 0)
+                data = self._read_nowait_chunk(ichar - offset if ichar else -1)
                 line.append(data)
                 line_size += len(data)
                 if ichar:
@@ -208,11 +254,7 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
                 break
 
             if not_enough:
-                self._waiter = self._create_waiter('readline')
-                try:
-                    yield from self._waiter
-                finally:
-                    self._waiter = None
+                yield from self._wait('readline')
 
         return b''.join(line)
 
@@ -235,7 +277,7 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
                         'might be infinite loop: \n%s', stack)
 
         if not n:
-            return EOF_MARKER
+            return b''
 
         if n < 0:
             # This used to just loop creating a new waiter hoping to
@@ -251,11 +293,7 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             return b''.join(blocks)
 
         if not self._buffer and not self._eof:
-            self._waiter = self._create_waiter('read')
-            try:
-                yield from self._waiter
-            finally:
-                self._waiter = None
+            yield from self._wait('read')
 
         return self._read_nowait(n)
 
@@ -265,26 +303,15 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
             raise self._exception
 
         if not self._buffer and not self._eof:
-            self._waiter = self._create_waiter('readany')
-            try:
-                yield from self._waiter
-            finally:
-                self._waiter = None
+            yield from self._wait('readany')
 
-        return self._read_nowait()
+        return self._read_nowait(-1)
 
     @asyncio.coroutine
     def readexactly(self, n):
         if self._exception is not None:
             raise self._exception
 
-        # There used to be "optimized" code here.  It created its own
-        # Future and waited until self._buffer had at least the n
-        # bytes, then called read(n).  Unfortunately, this could pause
-        # the transport if the argument was larger than the pause
-        # limit (which is twice self._limit).  So now we just read()
-        # into a local buffer.
-
         blocks = []
         while n > 0:
             block = yield from self.read(n)
@@ -297,7 +324,12 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
 
         return b''.join(blocks)
 
-    def read_nowait(self, n=None):
+    def read_nowait(self, n=-1):
+        # default was changed to be consistent with .read(-1)
+        #
+        # I believe the most users don't know about the method and
+        # they are not affected.
+        assert n is not None, "n should be -1"
         if self._exception is not None:
             raise self._exception
 
@@ -307,13 +339,10 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
 
         return self._read_nowait(n)
 
-    def _read_nowait(self, n=None):
-        if not self._buffer:
-            return EOF_MARKER
-
+    def _read_nowait_chunk(self, n):
         first_buffer = self._buffer[0]
         offset = self._buffer_offset
-        if n and len(first_buffer) - offset > n:
+        if n != -1 and len(first_buffer) - offset > n:
             data = first_buffer[offset:offset + n]
             self._buffer_offset += n
 
@@ -328,6 +357,19 @@ class StreamReader(asyncio.StreamReader, AsyncStreamReaderMixin):
         self._buffer_size -= len(data)
         return data
 
+    def _read_nowait(self, n):
+        chunks = []
+
+        while self._buffer:
+            chunk = self._read_nowait_chunk(n)
+            chunks.append(chunk)
+            if n != -1:
+                n -= len(chunk)
+                if n == 0:
+                    break
+
+        return b''.join(chunks) if chunks else b''
+
 
 class EmptyStreamReader(AsyncStreamReaderMixin):
 
@@ -355,22 +397,22 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
 
     @asyncio.coroutine
     def readline(self):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def read(self, n=-1):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def readany(self):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def readexactly(self, n):
         raise asyncio.streams.IncompleteReadError(b'', n)
 
     def read_nowait(self):
-        return EOF_MARKER
+        return b''
 
 
 class DataQueue:
@@ -428,7 +470,7 @@ class DataQueue:
                 raise self._exception
 
             assert not self._waiter
-            self._waiter = asyncio.Future(loop=self._loop)
+            self._waiter = helpers.create_future(self._loop)
             try:
                 yield from self._waiter
             except (asyncio.CancelledError, asyncio.TimeoutError):
@@ -446,10 +488,12 @@ class DataQueue:
                 raise EofStream
 
     if PY_35:
-        @asyncio.coroutine
         def __aiter__(self):
             return AsyncStreamIterator(self.read)
 
+        if not PY_352:  # pragma: no cover
+            __aiter__ = asyncio.coroutine(__aiter__)
+
 
 class ChunksQueue(DataQueue):
     """Like a :class:`DataQueue`, but for binary chunked data transfer."""
@@ -459,36 +503,26 @@ class ChunksQueue(DataQueue):
         try:
             return (yield from super().read())
         except EofStream:
-            return EOF_MARKER
+            return b''
 
     readany = read
 
 
 def maybe_resume(func):
 
-    @asyncio.coroutine
-    @functools.wraps(func)
-    def wrapper(self, *args, **kw):
-        result = yield from func(self, *args, **kw)
-
-        if self._stream.paused:
-            if self._buffer_size < self._b_limit:
-                try:
-                    self._stream.transport.resume_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = False
-        else:
-            if self._buffer_size > self._b_limit:
-                try:
-                    self._stream.transport.pause_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = True
-
-        return result
+    if asyncio.iscoroutinefunction(func):
+        @asyncio.coroutine
+        @functools.wraps(func)
+        def wrapper(self, *args, **kw):
+            result = yield from func(self, *args, **kw)
+            self._check_buffer_size()
+            return result
+    else:
+        @functools.wraps(func)
+        def wrapper(self, *args, **kw):
+            result = func(self, *args, **kw)
+            self._check_buffer_size()
+            return result
 
     return wrapper
 
@@ -507,6 +541,26 @@ class FlowControlStreamReader(StreamReader):
                 self._stream.transport.resume_reading()
             except (AttributeError, NotImplementedError):
                 pass
+            else:
+                self._stream.paused = False
+
+    def _check_buffer_size(self):
+        if self._stream.paused:
+            if self._buffer_size < self._b_limit:
+                try:
+                    self._stream.transport.resume_reading()
+                except (AttributeError, NotImplementedError):
+                    pass
+                else:
+                    self._stream.paused = False
+        else:
+            if self._buffer_size > self._b_limit:
+                try:
+                    self._stream.transport.pause_reading()
+                except (AttributeError, NotImplementedError):
+                    pass
+                else:
+                    self._stream.paused = True
 
     def feed_data(self, data, size=0):
         has_waiter = self._waiter is not None and not self._waiter.cancelled()
@@ -523,21 +577,29 @@ class FlowControlStreamReader(StreamReader):
                 self._stream.paused = True
 
     @maybe_resume
+    @asyncio.coroutine
     def read(self, n=-1):
         return (yield from super().read(n))
 
     @maybe_resume
+    @asyncio.coroutine
     def readline(self):
         return (yield from super().readline())
 
     @maybe_resume
+    @asyncio.coroutine
     def readany(self):
         return (yield from super().readany())
 
     @maybe_resume
+    @asyncio.coroutine
     def readexactly(self, n):
         return (yield from super().readexactly(n))
 
+    @maybe_resume
+    def read_nowait(self, n=-1):
+        return super().read_nowait(n)
+
 
 class FlowControlDataQueue(DataQueue):
     """FlowControlDataQueue resumes and pauses an underlying stream.
@@ -604,6 +666,6 @@ class FlowControlChunksQueue(FlowControlDataQueue):
         try:
             return (yield from super().read())
         except EofStream:
-            return EOF_MARKER
+            return b''
 
     readany = read
