@@ -1,6 +1,5 @@
 from .web_reqrep import Response
 
-
 __all__ = (
     'HTTPException',
     'HTTPError',
@@ -45,6 +44,7 @@ __all__ = (
     'HTTPPreconditionRequired',
     'HTTPTooManyRequests',
     'HTTPRequestHeaderFieldsTooLarge',
+    'HTTPUnavailableForLegalReasons',
     'HTTPServerError',
     'HTTPInternalServerError',
     'HTTPNotImplemented',
@@ -135,7 +135,7 @@ class _HTTPMove(HTTPRedirection):
             raise ValueError("HTTP redirects need a location to redirect to.")
         super().__init__(headers=headers, reason=reason,
                          body=body, text=text, content_type=content_type)
-        self.headers['Location'] = location
+        self.headers['Location'] = str(location)
         self.location = location
 
 
@@ -286,6 +286,17 @@ class HTTPRequestHeaderFieldsTooLarge(HTTPClientError):
     status_code = 431
 
 
+class HTTPUnavailableForLegalReasons(HTTPClientError):
+    status_code = 451
+
+    def __init__(self, link, *, headers=None, reason=None,
+                 body=None, text=None, content_type=None):
+        super().__init__(headers=headers, reason=reason,
+                         body=body, text=text, content_type=content_type)
+        self.headers['Link'] = '<%s>; rel="blocked-by"' % link
+        self.link = link
+
+
 ############################################################
 # 5xx Server Error
 ############################################################
