@@ -1,9 +1,10 @@
 import asyncio
+from unittest import mock
+
 import pytest
 
 from aiohttp import web
 from aiohttp.web_urldispatcher import View
-from unittest import mock
 
 
 def test_ctor():
@@ -12,7 +13,7 @@ def test_ctor():
     assert view.request is request
 
 
-@pytest.mark.run_loop
+@asyncio.coroutine
 def test_render_ok():
     resp = web.Response(text='OK')
 
@@ -27,31 +28,35 @@ def test_render_ok():
     assert resp is resp2
 
 
-@pytest.mark.run_loop
+@asyncio.coroutine
 def test_render_unknown_method():
 
     class MyView(View):
         @asyncio.coroutine
         def get(self):
             return web.Response(text='OK')
+        options = get
 
     request = mock.Mock()
     request.method = 'UNKNOWN'
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
         yield from MyView(request)
+    assert ctx.value.headers['allow'] == 'GET,OPTIONS'
     assert ctx.value.status == 405
 
 
-@pytest.mark.run_loop
+@asyncio.coroutine
 def test_render_unsupported_method():
 
     class MyView(View):
         @asyncio.coroutine
         def get(self):
             return web.Response(text='OK')
+        options = delete = get
 
     request = mock.Mock()
     request.method = 'POST'
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
         yield from MyView(request)
+    assert ctx.value.headers['allow'] == 'DELETE,GET,OPTIONS'
     assert ctx.value.status == 405
