@@ -40,7 +40,7 @@ class TestProxy(unittest.TestCase):
         tr, proto = mock.Mock(), mock.Mock()
         self.loop.create_connection = make_mocked_coro((tr, proto))
         conn = self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.path, 'http://www.python.org')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertIs(conn._transport, tr)
         self.assertIs(conn._protocol, proto)
 
@@ -62,6 +62,21 @@ class TestProxy(unittest.TestCase):
             "proxy_auth must be None or BasicAuth() tuple",
         )
 
+    @mock.patch('aiohttp.Request')
+    def test_connect_request_with_unicode_host(self, Request_mock):
+        loop = mock.Mock()
+        request = ClientRequest("CONNECT", URL("http://éé.com/"),
+                                loop=loop)
+
+        request.response_class = mock.Mock()
+        request.write_bytes = mock.Mock()
+        request.write_bytes.return_value = asyncio.Future(loop=loop)
+        request.write_bytes.return_value.set_result(None)
+        request.send(mock.Mock(), mock.Mock())
+
+        Request_mock.assert_called_with(mock.ANY, mock.ANY, "xn--9caa.com:80",
+                                        mock.ANY)
+
     def test_proxy_connection_error(self):
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro(
@@ -105,7 +120,7 @@ class TestProxy(unittest.TestCase):
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         conn = self.loop.run_until_complete(connector.connect(req))
 
-        self.assertEqual(req.path, 'http://www.python.org')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
@@ -148,7 +163,7 @@ class TestProxy(unittest.TestCase):
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         conn = self.loop.run_until_complete(connector.connect(req))
 
-        self.assertEqual(req.path, 'http://www.python.org')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
@@ -212,8 +227,8 @@ class TestProxy(unittest.TestCase):
 
         self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
-        self.assertEqual(proxy_req.path, 'www.python.org:443')
-        tr.pause_reading.assert_called_once_with()
+        self.assertEqual(proxy_req.url, URL('https://www.python.org'))
+        tr.close.assert_called_once_with()
         tr.get_extra_info.assert_called_with('socket', default=None)
 
         self.loop.run_until_complete(proxy_req.close())
@@ -341,7 +356,7 @@ class TestProxy(unittest.TestCase):
             loop=self.loop,
         )
         self.loop.run_until_complete(connector._create_connection(req))
-        self.assertEqual(req.path, 'http://localhost:1234/path')
+        self.assertEqual(req.url, URL('http://localhost:1234/path'))
 
     def test_proxy_auth_property(self):
         req = aiohttp.ClientRequest(
@@ -393,8 +408,8 @@ class TestProxy(unittest.TestCase):
 
         self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
-        self.assertEqual(proxy_req.path, 'www.python.org:443')
-        tr.pause_reading.assert_called_once_with()
+        self.assertEqual(proxy_req.url, URL('https://www.python.org'))
+        tr.close.assert_called_once_with()
         tr.get_extra_info.assert_called_with('socket', default=None)
 
         self.loop.run_until_complete(proxy_req.close())
@@ -494,7 +509,7 @@ class TestProxyConnector(unittest.TestCase):
         tr, proto = mock.Mock(), mock.Mock()
         self.loop.create_connection = make_mocked_coro((tr, proto))
         conn = self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.path, 'http://www.python.org')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertIs(conn._transport, tr)
         self.assertIs(conn._protocol, proto)
 
