@@ -1,11 +1,9 @@
-import pytest
+from unittest import mock
 
 from aiohttp import web
 from aiohttp.web_urldispatcher import View
-from unittest import mock
 
 
-@pytest.mark.run_loop
 async def test_render_ok():
     resp = web.Response(text='OK')
 
