@@ -2,7 +2,6 @@ import pytest
 
 from aiohttp import streams
 
-
 DATA = b'line1\nline2\nline3\n'
 
 
@@ -18,7 +17,6 @@ def create_stream(loop):
     return stream
 
 
-@pytest.mark.run_loop
 async def test_stream_reader_lines(loop):
     line_iter = iter(DATA.splitlines(keepends=True))
     async for line in create_stream(loop):
@@ -26,27 +24,24 @@ async def test_stream_reader_lines(loop):
     pytest.raises(StopIteration, next, line_iter)
 
 
-@pytest.mark.run_loop
 async def test_stream_reader_chunks_complete(loop):
     """Tests if chunked iteration works if the chunking works out
     (i.e. the data is divisible by the chunk size)
     """
     chunk_iter = chunkify(DATA, 9)
-    async for line in create_stream(loop).iter_chunked(9):
-        assert line == next(chunk_iter, None)
+    async for data in create_stream(loop).iter_chunked(9):
+        assert data == next(chunk_iter, None)
     pytest.raises(StopIteration, next, chunk_iter)
 
 
-@pytest.mark.run_loop
 async def test_stream_reader_chunks_incomplete(loop):
     """Tests if chunked iteration works if the last chunk is incomplete"""
     chunk_iter = chunkify(DATA, 8)
-    async for line in create_stream(loop).iter_chunked(8):
-        assert line == next(chunk_iter, None)
+    async for data in create_stream(loop).iter_chunked(8):
+        assert data == next(chunk_iter, None)
     pytest.raises(StopIteration, next, chunk_iter)
 
 
-@pytest.mark.run_loop
 async def test_data_queue_empty(loop):
     """Tests that async looping yields nothing if nothing is there"""
     buffer = streams.DataQueue(loop=loop)
@@ -56,7 +51,6 @@ async def test_data_queue_empty(loop):
         assert False
 
 
-@pytest.mark.run_loop
 async def test_data_queue_items(loop):
     """Tests that async looping yields objects identically"""
     buffer = streams.DataQueue(loop=loop)
@@ -72,8 +66,17 @@ async def test_data_queue_items(loop):
     pytest.raises(StopIteration, next, item_iter)
 
 
-@pytest.mark.run_loop
 async def test_stream_reader_iter_any(loop):
     it = iter([b'line1\nline2\nline3\n'])
     async for raw in create_stream(loop).iter_any():
         assert raw == next(it)
+    pytest.raises(StopIteration, next, it)
+
+
+async def test_stream_reader_iter(loop):
+    it = iter([b'line1\n',
+               b'line2\n',
+               b'line3\n'])
+    async for raw in create_stream(loop):
+        assert raw == next(it)
+    pytest.raises(StopIteration, next, it)
