@@ -9,7 +9,7 @@ import aiohttp
 from aiohttp import web, web_reqrep
 from aiohttp.test_utils import TestClient as _TestClient
 from aiohttp.test_utils import TestServer as _TestServer
-from aiohttp.test_utils import (AioHTTPTestCase, loop_context,
+from aiohttp.test_utils import (AioHTTPTestCase, RawTestServer, loop_context,
                                 make_mocked_request, setup_test_loop,
                                 teardown_test_loop, unittest_run_loop)
 
@@ -226,12 +226,10 @@ def test_make_mocked_request_transport():
 def test_test_client_props(loop):
     app = _create_example_app(loop)
     client = _TestClient(app, host='localhost')
-    assert client.app == app
     assert client.host == 'localhost'
     assert client.port is None
     with client:
         assert isinstance(client.port, int)
-        assert client.handler is not None
         assert client.server is not None
     assert client.port is None
 
@@ -278,3 +276,12 @@ def test_server_make_url_yarl_compatibility(loop):
             make_url('http://foo.com')
         with pytest.raises(AssertionError):
             make_url(URL('http://foo.com'))
+
+
+def test_raw_server_implicit_loop(loop):
+    @asyncio.coroutine
+    def handler(request):
+        pass
+    asyncio.set_event_loop(loop)
+    srv = RawTestServer(handler)
+    assert srv._loop is loop
