@@ -52,7 +52,7 @@ def test_handler_returns_not_response(loop, test_server, test_client):
     resp = yield from client.get('/')
     assert 500 == resp.status
 
-    logger.exception.assert_called_with("Error handling request")
+    assert logger.exception.called
 
 
 @asyncio.coroutine
@@ -825,19 +825,19 @@ def test_requests_count(loop, test_client):
     app = web.Application(loop=loop)
     app.router.add_get('/', handler)
     client = yield from test_client(app)
-    assert client.handler.requests_count == 0
+    assert client.server.handler.requests_count == 0
 
     resp = yield from client.get('/')
     assert 200 == resp.status
-    assert client.handler.requests_count == 1
+    assert client.server.handler.requests_count == 1
 
     resp = yield from client.get('/')
     assert 200 == resp.status
-    assert client.handler.requests_count == 2
+    assert client.server.handler.requests_count == 2
 
     resp = yield from client.get('/')
     assert 200 == resp.status
-    assert client.handler.requests_count == 3
+    assert client.server.handler.requests_count == 3
 
 
 @asyncio.coroutine
@@ -1185,3 +1185,26 @@ def test_custom_date_header(loop, test_client):
     resp = yield from client.get('/')
     assert 200 == resp.status
     assert resp.headers['Date'] == 'Sun, 30 Oct 2016 03:13:52 GMT'
+
+
+@asyncio.coroutine
+def test_response_task(loop, test_client):
+
+    srv_resp = None
+
+    @asyncio.coroutine
+    def handler(request):
+        nonlocal srv_resp
+        srv_resp = web.StreamResponse()
+        assert srv_resp.task is None
+        yield from srv_resp.prepare(request)
+        assert srv_resp.task is not None
+        return srv_resp
+
+    app = web.Application(loop=loop)
+    app.router.add_get('/', handler)
+    client = yield from test_client(app)
+
+    resp = yield from client.get('/')
+    assert 200 == resp.status
+    assert srv_resp.task is None
