@@ -3,10 +3,13 @@
 import base64
 import hashlib
 import os
+from unittest import mock
+
+import multidict
 import pytest
 
-from aiohttp import websocket, multidict, protocol, errors
-from unittest import mock
+from aiohttp import errors, protocol
+from aiohttp._ws_impl import WS_KEY, do_handshake
 
 
 @pytest.fixture()
@@ -23,82 +26,82 @@ def message():
 
 def gen_ws_headers(protocols=''):
     key = base64.b64encode(os.urandom(16)).decode()
-    hdrs = [('UPGRADE', 'websocket'),
-            ('CONNECTION', 'upgrade'),
-            ('SEC-WEBSOCKET-VERSION', '13'),
-            ('SEC-WEBSOCKET-KEY', key)]
+    hdrs = [('Upgrade', 'websocket'),
+            ('Connection', 'upgrade'),
+            ('Sec-Websocket-Version', '13'),
+            ('Sec-Websocket-Key', key)]
     if protocols:
-        hdrs += [('SEC-WEBSOCKET-PROTOCOL', protocols)]
+        hdrs += [('Sec-Websocket-Protocol', protocols)]
     return hdrs, key
 
 
 def test_not_get(message, transport):
     with pytest.raises(errors.HttpProcessingError):
-        websocket.do_handshake('POST', message.headers, transport)
+        do_handshake('POST', message.headers, transport)
 
 
 def test_no_upgrade(message, transport):
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
 
 def test_no_connection(message, transport):
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'keep-alive')])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'keep-alive')])
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
 
 def test_protocol_version(message, transport):
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'upgrade')])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'upgrade')])
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'upgrade'),
-                            ('SEC-WEBSOCKET-VERSION', '1')])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'upgrade'),
+                            ('Sec-Websocket-Version', '1')])
 
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
 
 def test_protocol_key(message, transport):
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'upgrade'),
-                            ('SEC-WEBSOCKET-VERSION', '13')])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'upgrade'),
+                            ('Sec-Websocket-Version', '13')])
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'upgrade'),
-                            ('SEC-WEBSOCKET-VERSION', '13'),
-                            ('SEC-WEBSOCKET-KEY', '123')])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'upgrade'),
+                            ('Sec-Websocket-Version', '13'),
+                            ('Sec-Websocket-Key', '123')])
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
     sec_key = base64.b64encode(os.urandom(2))
-    message.headers.extend([('UPGRADE', 'websocket'),
-                            ('CONNECTION', 'upgrade'),
-                            ('SEC-WEBSOCKET-VERSION', '13'),
-                            ('SEC-WEBSOCKET-KEY', sec_key.decode())])
+    message.headers.extend([('Upgrade', 'websocket'),
+                            ('Connection', 'upgrade'),
+                            ('Sec-Websocket-Version', '13'),
+                            ('Sec-Websocket-Key', sec_key.decode())])
     with pytest.raises(errors.HttpBadRequest):
-        websocket.do_handshake(message.method, message.headers, transport)
+        do_handshake(message.method, message.headers, transport)
 
 
 def test_handshake(message, transport):
     hdrs, sec_key = gen_ws_headers()
 
     message.headers.extend(hdrs)
-    status, headers, parser, writer, protocol = websocket.do_handshake(
+    status, headers, parser, writer, protocol = do_handshake(
         message.method, message.headers, transport)
     assert status == 101
     assert protocol is None
 
     key = base64.b64encode(
-        hashlib.sha1(sec_key.encode() + websocket.WS_KEY).digest())
+        hashlib.sha1(sec_key.encode() + WS_KEY).digest())
     headers = dict(headers)
-    assert headers['SEC-WEBSOCKET-ACCEPT'] == key.decode()
+    assert headers['Sec-Websocket-Accept'] == key.decode()
 
 
 def test_handshake_protocol(message, transport):
@@ -106,7 +109,7 @@ def test_handshake_protocol(message, transport):
     proto = 'chat'
 
     message.headers.extend(gen_ws_headers(proto)[0])
-    _, resp_headers, _, _, protocol = websocket.do_handshake(
+    _, resp_headers, _, _, protocol = do_handshake(
         message.method, message.headers, transport,
         protocols=[proto])
 
@@ -114,7 +117,7 @@ def test_handshake_protocol(message, transport):
 
     # also test if we reply with the protocol
     resp_headers = dict(resp_headers)
-    assert resp_headers['SEC-WEBSOCKET-PROTOCOL'] == proto
+    assert resp_headers['Sec-Websocket-Protocol'] == proto
 
 
 def test_handshake_protocol_agreement(message, transport):
@@ -124,7 +127,7 @@ def test_handshake_protocol_agreement(message, transport):
     server_protos = 'worse_proto,chat'
 
     message.headers.extend(gen_ws_headers(server_protos)[0])
-    _, resp_headers, _, _, protocol = websocket.do_handshake(
+    _, resp_headers, _, _, protocol = do_handshake(
         message.method, message.headers, transport,
         protocols=wanted_protos)
 
@@ -137,7 +140,7 @@ def test_handshake_protocol_unsupported(log, message, transport):
     message.headers.extend(gen_ws_headers('test')[0])
 
     with log('aiohttp.websocket') as ctx:
-        _, _, _, _, protocol = websocket.do_handshake(
+        _, _, _, _, protocol = do_handshake(
             message.method, message.headers, transport,
             protocols=[proto])
 
